@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 */
 
@@ -46,7 +46,7 @@
 #include <kapplication.h>
 #include <kcmdlineargs.h>
 #include <klocale.h>
-#include <klineeditdlg.h>
+#include <kinputdialog.h>
 #include <kstdaccel.h>
 #include <kfiledialog.h>
 #include <ksavefile.h>
@@ -110,7 +110,7 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	list<string> datadirs = PortableKDE::globalDirs("data");
 	list<string>::iterator it;
 
-	for(it = datadirs.begin(); it != datadirs.end(); it++)
+	for(it = datadirs.begin(); it != datadirs.end(); ++it)
 	{
 		string common = *it;
 		common += "artsbuilder";
@@ -135,14 +135,14 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 
 	}
 #endif
-	
+
 	arts_debug("PORT: structure");
 	structure = new Structure();
 	arts_debug("PORT: structure ok");
 	//ModuleList = structure->getModuleList();
 
 	mainDock = createDockWidget("mainDockWidget", 0, 0, "main_dock_widget");
-	
+
 	arts_debug("PORT: modulewidget");
 	modulewidget = new ModuleWidget(structure, mainDock, "mwidget");
 	mainDock->setWidget(modulewidget);
@@ -153,10 +153,10 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	mainDock->setDockSite(KDockWidget::DockCorner);
 	// forbit docking abilities of module widget itself
 	mainDock->setEnableDocking(KDockWidget::DockNone);
-	
+
 	setView(mainDock);
 	setMainDockWidget(mainDock);
-	
+
 	propertyDock = createDockWidget("propertyDock", 0, 0, i18n("Port Properties"));
 
 	propertyPanel = new PropertyPanel(propertyDock, "ppanel");
@@ -173,11 +173,11 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 			modulewidget,  SLOT  (selectPort(ModulePort *)));
 	connect(modulewidget,  SIGNAL(componentSelected(StructureComponent *)),
 			propertyPanel, SLOT  (setSelectedComponent(StructureComponent *)));
-	
+
 	// connection
 	connect(propertyPanel, SIGNAL(startConnection(ModulePort *)),
 			modulewidget,  SLOT  (startConnection(ModulePort *)));
-	
+
 	// port properties changed
 	connect(propertyPanel, SIGNAL(portPropertiesChanged(ModulePort *)),
 			modulewidget,  SLOT  (portPropertiesChanged(ModulePort *)));
@@ -185,9 +185,9 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	arts_debug("PORT: setcanvas");
 	structure->setCanvas(modulewidget);
 	arts_debug("PORT: setcanvas ok");
-	
+
 	mbroker_updateCount = 0;
-		
+
 	arts_debug("PORT: menumaker");
 	menumaker = new MenuMaker(new KActionMenu(i18n("Modules"), actionCollection(), "modulesmenu"));
 	//menumaker->addCategory("&Gui", "Gui_");
@@ -195,6 +195,7 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_ADD$");
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_AUTOPANNER$");
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_MUL$");
+	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_DIV$");
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_MULTI_ADD$");
 	menumaker->addCategory(i18n("&Synthesis/&Arithmetic + Mixing"), "Arts::Synth_XFADE$");
 	menumaker->addCategory(i18n("&Synthesis/&Busses"), "Arts::Synth_BUS_");
@@ -215,6 +216,7 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 	menumaker->addCategory(i18n("&Synthesis/&Midi + Sequencing"), "Arts::Synth_MIDI");
 	menumaker->addCategory(i18n("&Synthesis/&Midi + Sequencing"), "Arts::Interface_MIDI");
 	menumaker->addCategory(i18n("&Synthesis/&Midi + Sequencing"), "Arts::Synth_SEQUENCE$");
+	menumaker->addCategory(i18n("&Synthesis/&Midi + Sequencing"), "Arts::Synth_SEQUENCE_FREQ$");
 	menumaker->addCategory(i18n("&Synthesis/&Midi + Sequencing"), "Arts::Synth_STRUCT_KILL$");
 	menumaker->addCategory(i18n("&Synthesis/Sam&ples "), "Arts::Synth_PLAY_");
 	menumaker->addCategory(i18n("&Synthesis/&Sound IO"), "Arts::Synth_AMAN_");
@@ -280,7 +282,7 @@ ArtsBuilderWindow::ArtsBuilderWindow(const char *name)
 		connect(viewmenu, SIGNAL(aboutToShow()), this, SLOT(viewMenuAboutToShow()));
 	else
 		arts_debug("view menu not found!");
-	
+
 	m_filename = QString::null;
 	setModified(false);
 
@@ -291,7 +293,7 @@ void ArtsBuilderWindow::setupActions()
 {
 	// File menu
 	KStdAction::openNew(this, SLOT(fileNew()), actionCollection());
-	
+
 	(void)new KAction(i18n("Open Session..."), 0, this, SLOT(openSession()),
 					  actionCollection(), "file_open_session");
 	KStdAction::open(this, SLOT(open()), actionCollection());
@@ -305,7 +307,7 @@ void ArtsBuilderWindow::setupActions()
 					  actionCollection(), "file_execute_structure");
 	(void)new KAction(i18n("&Rename Structure..."), Qt::CTRL + Qt::Key_R, this, SLOT(rename()),
 					  actionCollection(), "file_rename_structure");
-	(void)new KAction(i18n("&Publish Structure!"), Qt::CTRL + Qt::Key_P, this, SLOT(publish()),
+	(void)new KAction(i18n("&Publish Structure"), Qt::CTRL + Qt::Key_P, this, SLOT(publish()),
 					  actionCollection(), "file_publish_structure");
 	KStdAction::quit(this, SLOT(close()), actionCollection());
 
@@ -345,7 +347,7 @@ void ArtsBuilderWindow::setupActions()
 void ArtsBuilderWindow::fillModuleMenu()
 {
 	long updateCount = 3;  /* PORT: automatic update of menues missing */
-	
+
 	if(updateCount != mbroker_updateCount)
 	{
 		mbroker_updateCount = updateCount;
@@ -359,12 +361,12 @@ void ArtsBuilderWindow::fillModuleMenu()
 
 		vector<Arts::TraderOffer>::iterator i;
 		long n = 1;		/* TODO:PORT: is this necessary? I think not */
-		for(i = offers->begin(); i != offers->end(); i++)
+		for(i = offers->begin(); i != offers->end(); ++i)
 		{
 			Arts::TraderOffer& offer = *i;
 			string name = offer.interfaceName();
 			menumaker->addItem(name.c_str(),n++);
-			
+
 			/* PORT: templates missing
 			if(strncmp(name, "template_", strlen("template_")) == 0)
 			{
@@ -400,7 +402,7 @@ void ArtsBuilderWindow::fillModuleMenu()
 			{
 				const char *name = (*Modules)[i];
 				menumaker->addItem(name, i);
-			
+
 				if(strncmp(name, "template_", strlen("template_")) == 0)
 				{
 					char *xname = strdup(&name[strlen("template_")]);
@@ -439,8 +441,7 @@ void ArtsBuilderWindow::publish()
 	checkName();
 	structure->publish();
 	KMessageBox::information(this,
-		QString(i18n("The structure has been published as: '"))
-		 + QString(structure->name().c_str()) + QString(i18n("' on the server.")));
+		i18n("The structure has been published as: '%1' on the server.").arg( structure->name().c_str() ));
 }
 
 QString ArtsBuilderWindow::getOpenFilename(const char *pattern, const char *initialDir)
@@ -497,8 +498,8 @@ void ArtsBuilderWindow::open(QString filename)
 		if(!structure->valid())
 		{
 			KMessageBox::sorry(this,
-			i18n("The structure couldn't be loaded correctly. Maybe some of\n"
-				 "the modules used in the file aren't available in this\n"
+			i18n("The structure could not be loaded correctly. Maybe some of\n"
+				 "the modules used in the file are not available in this\n"
 				 "version of aRts."),
 				i18n("Arts Warning"));
 		}
@@ -548,8 +549,8 @@ void ArtsBuilderWindow::openExample()
 	if(!dir)
 		KMessageBox::sorry(
 			this,
-			i18n("Unable to find the examples directory.\nUsing the current directory instead."),
-			i18n("Arts Warning"));
+			i18n("Unable to find the examples folder.\nUsing the current folder instead."),
+			i18n("aRts Warning"));
 
 	open(getOpenFilename("*.arts", QFile::encodeName(dir)));
 }
@@ -587,17 +588,17 @@ bool ArtsBuilderWindow::save(QString filename)
 		KMessageBox::sorry(this,
 			i18n("The file '%1' could not be opened for writing: %2")
 			.arg(filename).arg(strerror(file.status())),
-			i18n("Arts Warning"));
+			i18n("aRts Warning"));
 		return false;
 	}
 
 	structure->saveInto(file.fstream());
-	
+
 	if(!file.close()) {
 		KMessageBox::sorry(this,
 			i18n("Saving to file '%1' could not be finished correctly: %2")
 			.arg(filename).arg(strerror(file.status())),
-			i18n("Arts Warning"));
+			i18n("aRts Warning"));
 		return false;
 	}
 
@@ -626,14 +627,16 @@ void ArtsBuilderWindow::checkName()
 
 void ArtsBuilderWindow::rename()
 {
-	KLineEditDlg dlg(i18n("Enter structure name:"), structure->name().c_str(), this);
-	dlg.setCaption(i18n("Rename Structure"));
-	if (dlg.exec())
+    bool ok;
+
+    QString name = KInputDialog::getText( i18n( "Rename Structure" ),
+      i18n( "Enter structure name:" ), structure->name().c_str(), &ok, this );
+	if (ok)
 	{
 		arts_debug("rename OK...");
-		structure->rename(dlg.text().local8Bit());
+		structure->rename(name.local8Bit());
 	}
-	
+
 	setModified(true);
 }
 
@@ -681,8 +684,8 @@ void ArtsBuilderWindow::execute()
 		execDlg = 0;
 
 		KMessageBox::sorry(this,
-			i18n("Couldn't execute your structure. Make sure that the\n"
-				 "sound server (artsd) is running.\n"), i18n("Arts Warning"));
+			i18n("Could not execute your structure. Make sure that the\n"
+				 "sound server (artsd) is running.\n"), i18n("aRts Warning"));
 	}
 }
 
@@ -751,7 +754,7 @@ void ArtsBuilderWindow::addInterface()
 	ifd->exec();
 
 	Arts::ModuleInfo minfo = makeModuleInfo(ifd->interfaceName());
-	if(minfo.name != "")
+	if(!minfo.name.empty())
 		modulewidget->addInterface(minfo);
 
 	delete ifd;
@@ -784,7 +787,7 @@ void ArtsBuilderWindow::addModule(const char *name)
 	arts_debug("addModule(%s)", name);
 	Arts::ModuleInfo minfo = makeModuleInfo(name);
 
-	if(minfo.name != "")
+	if(!minfo.name.empty())
 		modulewidget->addModule(minfo);
 #if 0
 	const char *name = menumaker->findID(module);
@@ -869,8 +872,8 @@ ArtsBuilderApp::ArtsBuilderApp(QString filename)
 		mainWindow->open(filename);
 	} else {
 		KMessageBox::sorry(0,
-						   QString(i18n("The specified file '%1' does not exist.")).arg(filename),
-						   i18n("Arts Warning"));
+		   i18n("The specified file '%1' does not exist.").arg(filename),
+		   i18n("aRts Warning"));
 	}
 }
 
@@ -902,8 +905,8 @@ void ArtsBuilderApp::end()
 
 static KCmdLineOptions options[] =
 {
-	{ "+[file]", I18N_NOOP("Optional .arts file to be loaded."), 0 },
-	{ 0, 0, 0 }
+	{ "+[file]", I18N_NOOP("Optional .arts file to be loaded"), 0 },
+	KCmdLineLastOption
 };
 
 #ifdef COMMON_BINARY
@@ -915,7 +918,7 @@ int main(int argc, char **argv)
 	KAboutData aboutData("artsbuilder",
 						 I18N_NOOP("artsbuilder"),
 						 ARTS_VERSION,
-						 I18N_NOOP("Arts synthesizer designer"),
+						 I18N_NOOP("aRts synthesizer designer"),
 						 KAboutData::License_GPL,
 						 "(C) 1998-2001, Stefan Westerfeld",
 						 I18N_NOOP("The analog real-time synthesizer graphical design tool."),
