@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
     */
 
@@ -30,6 +30,7 @@
 #include <kapplication.h>
 #include <kseparator.h>
 #include <klocale.h>
+#include <kstdguiitem.h>
 #include <stdio.h>
 #include <set>
 #include <arts/debug.h>
@@ -37,7 +38,7 @@
 
 static void min_size(QWidget *w) {
   w->setMinimumSize(w->sizeHint());
-} 
+}
 
 RetrieveDlg::RetrieveDlg(QWidget *parent) :QDialog(parent,"X", TRUE)
 {
@@ -80,7 +81,7 @@ RetrieveDlg::RetrieveDlg(QWidget *parent) :QDialog(parent,"X", TRUE)
 		names.insert(structures[i]->Name());
 
 	set<string>::iterator ni;
-	for(ni=names.begin();ni!=names.end();ni++)
+	for(ni=names.begin();ni!=names.end();++ni)
 		listbox->insertItem((*ni).c_str());
 #endif
 
@@ -103,13 +104,13 @@ RetrieveDlg::RetrieveDlg(QWidget *parent) :QDialog(parent,"X", TRUE)
 	buttonlayout->addSpacing(5);
 	KButtonBox *bbox = new KButtonBox(this);
 
-	bbox->addButton(i18n("&Help"), this, SLOT( help() ));
+	bbox->addButton(KStdGuiItem::help(), this, SLOT( help() ));
 	bbox->addStretch(1);
 
-	QButton *cancelbutton = bbox->addButton(i18n("&Cancel"));
+	QButton *cancelbutton = bbox->addButton(KStdGuiItem::cancel());
 	connect( cancelbutton, SIGNAL( clicked() ), SLOT(reject() ) );
 
-	QButton *okbutton = bbox->addButton(i18n("&OK"));
+	QButton *okbutton = bbox->addButton(KStdGuiItem::ok());
 	connect( okbutton, SIGNAL( clicked() ), SLOT(accept() ) );
 
 	bbox->layout();
