@@ -37,6 +37,7 @@
 #include <artsflow.h>
 #include <connect.h>
 #include <artsmoduleseffects.h>
+#include <kdelibs_export.h>
 
 #include "artsactions.h"
 
@@ -44,7 +45,7 @@
 
 class VolumeSlider;
 
-class ArtsControlAppletPrivate : public QObject {
+class KDE_EXPORT ArtsControlAppletPrivate : public QObject {
    Q_OBJECT
 private:
 	ArtsControlApplet *_parent;
