@@ -21,6 +21,7 @@
 #include <kiconloader.h>
 #include <kconfig.h>
 #include <kaction.h>
+#include <kurl.h>
 
 #include "collectionlist.h"
 #include "playlistcollection.h"
@@ -30,6 +31,7 @@
 #include "actioncollection.h"
 #include "tag.h"
 #include "viewmode.h"
+#include "coverinfo.h"
 
 using namespace ActionCollection;
 
@@ -84,6 +86,9 @@ void CollectionList::initialize(PlaylistCollection *collection)
 
 PlaylistItem *CollectionList::createItem(const FileHandle &file, QListViewItem *, bool)
 {
+    // It's probably possible to optimize the line below away, but, well, right
+    // now it's more important to not load duplicate items.
+
     if(m_itemsDict.find(file.absFilePath()))
 	return 0;
 
@@ -124,9 +129,52 @@ void CollectionList::setupTreeViewEntries(ViewMode *viewMode) const
     columnList << PlaylistItem::GenreColumn;
     columnList << PlaylistItem::AlbumColumn;
 
-    for(QValueList<int>::Iterator colIt = columnList.begin(); colIt != columnList.end(); ++colIt)
+    QStringList items;
+    for(QValueList<int>::Iterator colIt = columnList.begin(); colIt != columnList.end(); ++colIt) {
+	items.clear();
 	for(TagCountDictIterator it(*m_columnTags[*colIt]); it.current(); ++it)
-	    treeViewMode->slotAddItem(it.currentKey(), *colIt);
+	    items << it.currentKey();
+
+	treeViewMode->slotAddItems(items, *colIt);
+    }
+}
+
+void CollectionList::slotNewItems(const KFileItemList &items)
+{
+    QStringList files;
+
+    for(KFileItemListIterator it(items); it.current(); ++it)
+	files.append((*it)->url().path());
+
+    addFiles(files);
+    update();
+}
+
+void CollectionList::slotRefreshItems(const KFileItemList &items)
+{
+    for(KFileItemListIterator it(items); it.current(); ++it) {
+	CollectionListItem *item = lookup((*it)->url().path());
+
+	if(item) {
+	    item->refreshFromDisk();
+
+	    // If the item is no longer on disk, remove it from the collection.
+
+	    if(item->file().fileInfo().exists())
+		item->repaint();
+	    else
+		delete item;
+	}
+    }
+
+    update();
+}
+
+void CollectionList::slotDeleteItem(KFileItem *item)
+{
+    CollectionListItem *listItem = lookup(item->url().path());
+    if(listItem)
+	clearItem(listItem);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -180,11 +228,7 @@ CollectionList::CollectionList(PlaylistCollection *collection) :
 {
     new KAction(i18n("Show Playing"), KShortcut(), actions(), "showPlaying");
 
-    m_dirWatch = new KDirWatch;
-    connect(m_dirWatch, SIGNAL(deleted(const QString &)), this, SLOT(slotRemoveItem(const QString &)));
-    connect(m_dirWatch, SIGNAL(dirty(const QString &)), this, SLOT(slotRefreshItem(const QString &)));
     connect(action("showPlaying"), SIGNAL(activated()), this, SLOT(slotShowPlaying()));
-    m_dirWatch->startScan();
 
     connect(action<KToolBarPopupAction>("back")->popupMenu(), SIGNAL(aboutToShow()),
 	    this, SLOT(slotPopulateBackMenu()));
@@ -217,16 +261,14 @@ CollectionList::~CollectionList()
     clearItems(items());
     for(TagCountDicts::Iterator it = m_columnTags.begin(); it != m_columnTags.end(); ++it)
 	delete *it;
-
-    delete m_dirWatch;
 }
 
 void CollectionList::contentsDropEvent(QDropEvent *e)
 {
     if(e->source() == this)
 	return;
-    else
-	decode(e);
+    else 
+	decode(e, static_cast<PlaylistItem *>(itemAt(contentsToViewport(e->pos()))));
 }
 
 void CollectionList::contentsDragMoveEvent(QDragMoveEvent *e)
@@ -310,6 +352,7 @@ void CollectionListItem::refresh()
 {
     int offset = static_cast<Playlist *>(listView())->columnOffset();
     int columns = lastColumn() + offset + 1;
+
     data()->local8Bit.resize(columns);
     data()->cachedWidths.resize(columns);
 
@@ -344,21 +387,27 @@ void CollectionListItem::refresh()
 	    playlist()->slotWeightDirty(i);
     }
 
-    playlist()->dataChanged();
-    CollectionList::instance()->dataChanged();
-    emit CollectionList::instance()->signalCollectionChanged();
+    file().coverInfo()->setCover();
 
     if(listView()->isVisible())
 	repaint();
 
     for(PlaylistItemList::Iterator it = m_children.begin(); it != m_children.end(); ++it) {
+	(*it)->playlist()->update();
+	(*it)->playlist()->dataChanged();
 	if((*it)->listView()->isVisible())
 	    (*it)->repaint();
     }
+
+    CollectionList::instance()->dataChanged();
+    emit CollectionList::instance()->signalCollectionChanged();
 }
 
-PlaylistItem *CollectionListItem::itemForPlaylist(const Playlist *playlist) const
+PlaylistItem *CollectionListItem::itemForPlaylist(const Playlist *playlist)
 {
+    if(playlist == CollectionList::instance())
+	return this;
+
     PlaylistItemList::ConstIterator it;
     for(it = m_children.begin(); it != m_children.end(); ++it)
 	if((*it)->playlist() == playlist)
@@ -377,6 +426,13 @@ void CollectionListItem::updateCollectionDict(const QString &oldPath, const QStr
     collection->addToDict(newPath, this);
 }
 
+void CollectionListItem::repaint() const
+{
+    QListViewItem::repaint();
+    for(PlaylistItemList::ConstIterator it = m_children.begin(); it != m_children.end(); ++it)
+	(*it)->repaint();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // CollectionListItem protected methods
 ////////////////////////////////////////////////////////////////////////////////
