@@ -1,8 +1,6 @@
 /***************************************************************************
-                          main.cpp  -  description
-                             -------------------
     begin                : Mon Feb  4 23:40:41 EST 2002
-    copyright            : (C) 2002 by Scott Wheeler
+    copyright            : (C) 2002 - 2004 by Scott Wheeler
     email                : wheeler@kde.org
 ***************************************************************************/
 
@@ -19,11 +17,14 @@
 #include <kcmdlineargs.h>
 #include <kaboutdata.h>
 #include <dcopclient.h>
+#include <kconfigbase.h>
+#include <kconfig.h>
 
 #include "juk.h"
 
 static const char description[] = I18N_NOOP("Jukebox and music manager for KDE");
 static const char scott[]       = I18N_NOOP("Author, chief dork and keeper of the funk");
+static const char michael[]     = I18N_NOOP("Assistant super-hero, fixer of many things");
 static const char daniel[]      = I18N_NOOP("System tray docking, \"inline\" tag editing,\nbug fixes, evangelism, moral support");
 static const char tim[]         = I18N_NOOP("GStreamer port");
 static const char stefan[]      = I18N_NOOP("Global keybindings support");
@@ -34,12 +35,11 @@ static const char adam[]        = I18N_NOOP("Co-conspirator in MusicBrainz wizar
 static const char matthias[]    = I18N_NOOP("Friendly, neighborhood aRts guru");
 static const char maks[]        = I18N_NOOP("Making JuK friendlier to people with\nterabytes of music");
 static const char antonio[]     = I18N_NOOP("DCOP interface");
+static const char allan[]       = I18N_NOOP("FLAC and MPC support");
+static const char nathan[]      = I18N_NOOP("Album cover manager");
 
 static KCmdLineOptions options[] =
 {
-#ifndef NO_DEBUG
-    { "norestore", I18N_NOOP("Restore playlists.  Use --norestore for debugging."), 0 },
-#endif
     { "+[file(s)]", I18N_NOOP("File(s) to open"), 0 },
     KCmdLineLastOption
 };
@@ -47,11 +47,12 @@ static KCmdLineOptions options[] =
 int main(int argc, char *argv[])
 {
     KAboutData aboutData("juk", I18N_NOOP("JuK"),
-                         "2.0.2", description, KAboutData::License_GPL,
-                         "(c) 2002, 2003, Scott Wheeler", 0,
-			 "http://developer.kde.org/~wheeler/juk.html");
+                         "2.2.1", description, KAboutData::License_GPL,
+                         "(c) 2002 - 2005, Scott Wheeler", 0,
+                         "http://developer.kde.org/~wheeler/juk.html");
 
     aboutData.addAuthor("Scott Wheeler", scott, "wheeler@kde.org");
+    aboutData.addAuthor("Michael Pyne", michael, "michael.pyne@kdemail.net");
     aboutData.addCredit("Daniel Molkentin", daniel, "molkentin@kde.org");
     aboutData.addCredit("Tim Jansen", tim, "tim@tjansen.de");
     aboutData.addCredit("Stefan Asserhäll", stefan, "stefan.asserhall@telia.com");
@@ -62,6 +63,8 @@ int main(int argc, char *argv[])
     aboutData.addCredit("Matthias Kretz", matthias, "kretz@kde.org");
     aboutData.addCredit("Maks Orlovich", maks, "maksim@kde.org");
     aboutData.addCredit("Antonio Larrosa Jimenez", antonio, "larrosa@kde.org");
+    aboutData.addCredit("Allan Sandfeld Jensen", allan, "kde@carewolf.com");
+    aboutData.addCredit("Nathan Toone", nathan, "nathan@toonetown.com");
 
     KCmdLineArgs::init(argc, argv, &aboutData);
     KCmdLineArgs::addCmdLineOptions(options);
@@ -74,20 +77,18 @@ int main(int argc, char *argv[])
     // before JuK has finished its initialization.
 
     a.dcopClient()->suspend();
-    JuK *juk = new JuK();
+    JuK *juk = new JuK;
     a.dcopClient()->resume();
 
     a.setMainWidget(juk);
 
     bool startDocked;
 
-    KConfig *config = KGlobal::config();
-    {
-        KConfigGroupSaver saver(config, "Settings");
-	startDocked = config->readBoolEntry("StartDocked", false);
-    }
+    KConfigGroup config(KGlobal::config(), "Settings");
+    startDocked = config.readBoolEntry("StartDocked", false);
+
     if(!startDocked)
-	juk->show();
+        juk->show();
 
     return a.exec();
 }
