@@ -87,11 +87,12 @@ signals:
 private:
     Player *player() const;
     void setup();
-
+    void setOutput(const QString &);
+    
 private slots:
     void slotPollPlay();
     void slotUpdateTime(int position);
-    void slotSetOutput(int system);
+    void slotSetOutput(const QString &);
     void slotSetVolume(int volume);
 
 private:
