@@ -1,8 +1,6 @@
 /***************************************************************************
-                          playlist.h  -  description
-                             -------------------
     begin                : Sat Feb 16 2002
-    copyright            : (C) 2002, 2003 by Scott Wheeler
+    copyright            : (C) 2002 - 2004 by Scott Wheeler
     email                : wheeler@kde.org
 ***************************************************************************/
 
@@ -23,47 +21,71 @@
 #include <kdebug.h>
 #include <kglobalsettings.h>
 
-#include <qstringlist.h>
 #include <qvaluevector.h>
-#include <qptrstack.h>
 #include <qfileinfo.h>
 
 #include "stringhash.h"
 #include "playlistsearch.h"
 #include "tagguesser.h"
+#include "playlistinterface.h"
+#include "playlistitem.h"
 
 class KPopupMenu;
 class KActionMenu;
 
 class QEvent;
 
-class PlaylistSearch;
+class PlaylistCollection;
+
+class PlaylistToolTip;
+class UpcomingPlaylist;
 
-class PlaylistItem;
 typedef QValueList<PlaylistItem *> PlaylistItemList;
 
-typedef QValueList<Playlist *> PlaylistList;
-
-class Playlist : public KListView
+class Playlist : public KListView, public PlaylistInterface
 {
     Q_OBJECT
 
 public:
 
-    /**
-     * Before creating a playlist directly, please see
-     * PlaylistSplitter::createPlaylist().
-     */
-    Playlist(QWidget *parent, const QString &name = QString::null);
+    Playlist(PlaylistCollection *collection, const QString &name = QString::null,
+	     const QString &iconName = "midi");
+    Playlist(PlaylistCollection *collection, const PlaylistItemList &items,
+	     const QString &name = QString::null, const QString &iconName = "midi");
+    Playlist(PlaylistCollection *collection, const QFileInfo &playlistFile,
+	     const QString &iconName = "midi");
 
     /**
-     * Before creating a playlist directly, please see
-     * PlaylistSplitter::openPlaylist().
+     * This constructor should generally only be used either by the cache
+     * restoration methods or by subclasses that want to handle calls to
+     * PlaylistCollection::setupPlaylist() differently.
      */
-    Playlist(const QFileInfo &playlistFile, QWidget *parent, const QString &name);
+    Playlist(PlaylistCollection *collection, bool delaySetup);
 
     virtual ~Playlist();
 
+
+    // The following group of functions implement the PlaylistInterface API.
+
+    virtual QString name() const;
+    virtual FileHandle currentFile() const;
+    virtual int count() const { return childCount(); }
+    virtual int time() const;
+    virtual void playNext();
+    virtual void playPrevious();
+    virtual void stop();
+
+    /**
+     * Plays the top item of the playlist.
+     */
+    void playFirst();
+
+    /**
+     * Plays the next album in the playlist.  Only useful when in album random
+     * play mode.
+     */
+    void playNextAlbum();
+
     /**
      * Saves the file to the currently set file name.  If there is no filename
      * currently set, the default behavior is to prompt the user for a file
@@ -86,9 +108,6 @@ public:
      * preferable to delay this notification until after other operations have
      * completed.  In those cases set \a emitChanged to false and call the
      * signal directly.
-     *
-     * @see signalCountChanged()
-     * @see emitCountChanged()
      */
     virtual void clearItem(PlaylistItem *item, bool emitChanged = true);
 
@@ -99,17 +118,25 @@ public:
     virtual void clearItems(const PlaylistItemList &items);
 
     /**
-     * All of the (media) files in the list.
+     * Accessor function to return a pointer to the currently playing file.
+     *
+     * @return 0 if no file is playing, otherwise a pointer to the PlaylistItem
+     *     of the track that is currently playing.
      */
-    QStringList files();
+    static PlaylistItem *playingItem();
 
     /**
-     * Returns a list of all of the \e visible items in the playlist.
+     * All of the (media) files in the list.
+     */
+    QStringList files() const;
+
+    /**
+     * Returns a list of all of the items in the playlist.
      */
     virtual PlaylistItemList items();
 
     /**
-     * Returns a list of all of the items in the playlist.
+     * Returns a list of all of the \e visible items in the playlist.
      */
     PlaylistItemList visibleItems();
 
@@ -119,9 +146,9 @@ public:
     PlaylistItemList selectedItems();
 
     /**
-     * Returns a list of the last 10 played items.
+     * Returns properly casted first child item in list.
      */
-    PlaylistItemList historyItems(PlaylistItem *current, bool random) const;
+    PlaylistItem *firstChild() const;
 
     /**
      * Allow duplicate files in the playlist.
@@ -133,8 +160,7 @@ public:
      * of PlaylistItems virtual.  In this case it allows for the creation of
      * both PlaylistItems and CollectionListItems.
      */
-    virtual PlaylistItem *createItem(const QFileInfo &file,
-				     const QString &absFilePath = QString::null,
+    virtual PlaylistItem *createItem(const FileHandle &file,
 				     QListViewItem *after = 0,
 				     bool emitChanged = true);
 
@@ -146,12 +172,22 @@ public:
      * be the same.     
      */
     template <class ItemType, class CollectionItemType, class CollectionListType>
-    ItemType *createItem(const QFileInfo &file,
-			 const QString &absFilePath = QString::null,
+    ItemType *createItem(const FileHandle &file,
 			 QListViewItem *after = 0,
 			 bool emitChanged = true);
 
-    virtual void createItems(const PlaylistItemList &siblings);
+    virtual void createItems(const PlaylistItemList &siblings, PlaylistItem *after = 0);
+
+    /**
+     * This handles adding files of various types -- music, playlist or directory
+     * files.  Music files that are found will be added to this playlist.  New
+     * playlist files that are found will result in new playlists being created.
+     *
+     * Note that this should not be used in the case of adding *only* playlist
+     * items since it has the overhead of checking to see if the file is a playlist
+     * or directory first.
+     */
+    virtual void addFiles(const QStringList &files, PlaylistItem *after = 0);
 
     /**
      * Returns the file name associated with this playlist (an m3u file) or
@@ -166,53 +202,25 @@ public:
     void setFileName(const QString &n) { m_fileName = n; }
 
     /**
-     * Hides column \a c.  If \a emitChanged is true then a signal that the
+     * Hides column \a c.  If \a updateSearch is true then a signal that the
      * visible columns have changed will be emitted and things like the search
      * will be udated.
      */
-    void hideColumn(int c, bool emitChanged = true);
+    void hideColumn(int c, bool updateSearch = true);
 
     /**
-     * Shows column \a c.  If \a emitChanged is true then a signal that the
+     * Shows column \a c.  If \a updateSearch is true then a signal that the
      * visible columns have changed will be emitted and things like the search
      * will be udated.
      */
-    void showColumn(int c, bool emitChanged = true);
+    void showColumn(int c, bool updateSearch = true);
     bool isColumnVisible(int c) const;
 
-    /**
-     * If m_playlistName has no value -- i.e. the name has not been set to
-     * something other than the filename, this returns the filename less the
-     * extension.  If m_playlistName does have a value, this returns that.
-     */
-    QString name() const;
-
     /**
      * This sets a name for the playlist that is \e different from the file name.
      */
     void setName(const QString &n);
 
-    /**
-     * Returns the number of items in the playlist.
-     */
-    int count() const { return childCount(); }
-
-    /**
-     * Returns the next item to be played.  If random is false this is just
-     * the next item in the playlist (or null if the current items is the last
-     * item in the list).  If random is true, then it will select an item at
-     * random from this list (and try to be a bit clever about it to not repeat
-     * items before everything has been played at least once).
-     */
-    PlaylistItem *nextItem(PlaylistItem *current, bool random = false);
-
-    /**
-     * Returns the item played before the currently playing item.  If random is
-     * false, this is simply the item above the currently playing item in the
-     * list.  If random is true this checks the history of recently played items.
-     */
-    PlaylistItem *previousItem(PlaylistItem *current, bool random = false);
-
     /**
      * Returns the KActionMenu that allows this to be embedded in menus outside
      * of the playlist.
@@ -220,9 +228,10 @@ public:
     KActionMenu *columnVisibleAction() const { return m_columnVisibleAction; }
 
     /**
-     * Set item to be the playing item; also set this list to be the playing list.
+     * Set item to be the playing item.  If \a item is null then this will clear
+     * the playing indicator.
      */
-    static void setPlaying(PlaylistItem *item, bool p = true);
+    static void setPlaying(PlaylistItem *item, bool addToHistory = true);
 
     /**
      * Returns true if this playlist is currently playing.
@@ -235,6 +244,11 @@ public:
      */
     void updateLeftColumn();
 
+    /**
+     * Returns the leftmost visible column of the listview.
+     */
+    int leftColumn() const { return m_leftColumn; }
+
     /**
      * Sets the items in the list to be either visible based on the value of
      * visible.  This is useful for search operations and such.
@@ -249,19 +263,14 @@ public:
 
     /**
      * Set the search associtated with this playlist.
-     *
-     * \note This does not cause the search to be rerun.
      */
-    void setSearch(const PlaylistSearch &s) { m_search = s; }
+    void setSearch(const PlaylistSearch &s);
 
     /**
-     * Emits a signal indicating that the number of items have changed.  This
-     * is useful in conjunction with createItem() where emitChanged is false.
-     *
-     * In many situations it is not practical for speed reasons to trigger the
-     * actions associated with signalCountChanged() after each insertion.
+     * If the search is disabled then all items will be shown, not just those that
+     * match the current search.
      */
-    void emitCountChanged() { emit signalCountChanged(this); }
+    void setSearchEnabled(bool searchEnabled);
 
     /**
      * Marks \a item as either selected or deselected based.
@@ -283,7 +292,23 @@ public:
      */
     virtual bool readOnly() const { return false; }
 
-    void setColumnWidthUpdatesDisabled(bool disabled) { m_disableColumnWidthUpdates = disabled; }
+    /**
+     * Returns true if it's possible to reload this playlist.
+     */
+    virtual bool canReload() const { return !m_fileName.isNull(); }
+
+    /**
+     * Returns true if the playlist is a search playlist and the search should be
+     * editable.
+     */
+    virtual bool searchIsEditable() const { return false; }
+
+    /**
+     * Synchronizes the the playing item in this playlist with the playing item
+     * in \a sources.  If \a setMaster is true, this list will become the source
+     * for determining the next item.
+     */
+    void synchronizePlayingItems(const PlaylistList &sources, bool setMaster);
 
     /**
      * Playlists have a common set of shared settings such as visible columns
@@ -292,17 +317,14 @@ public:
      */
     void applySharedSettings();
 
+    void read(QDataStream &s);
+
 public slots:
     /**
      * Remove the currently selected items from the playlist and disk.
      */
     void slotRemoveSelectedItems() { removeFromDisk(selectedItems()); };
 
-    /**
-     * Set the first selected item to be the next item returned by nextItem().
-     */
-    void slotSetNext();
-
     /*
      * The edit slots are required to use the canonical names so that they are
      * detected by the application wide framework.
@@ -344,6 +366,22 @@ public slots:
      */
     void slotRenameFile();
 
+    /**
+     * Sets the cover of the selected items, pass in true if you want to load from the local system,
+     * false if you want to load from the internet.
+     */
+    void slotAddCover(bool fromLocal);
+
+    /**
+     * Shows a large image of the cover
+     */
+    void slotViewCover();
+
+    /**
+     * Removes covers from the selected items
+     */
+    void slotRemoveCover();
+
     /**
      * Reload the playlist contents from the m3u file.
      */
@@ -356,6 +394,12 @@ public slots:
      */
     void slotWeightDirty(int column = -1);
 
+    void slotShowPlaying();
+
+    void slotColumnResizeModeChanged();
+
+    virtual void dataChanged();
+
 protected:
     /**
      * Remove \a items from the playlist and disk.  This will ignore items that
@@ -369,13 +413,19 @@ protected:
     virtual QDragObject *dragObject(QWidget *parent);
     virtual QDragObject *dragObject() { return dragObject(this); }
     virtual bool canDecode(QMimeSource *s);
-    virtual void decode(QMimeSource *s, PlaylistItem *after = 0);
+    virtual void decode(QMimeSource *s, PlaylistItem *item = 0);
     virtual void contentsDropEvent(QDropEvent *e);
+    virtual void contentsMouseDoubleClickEvent(QMouseEvent *e);
     virtual void showEvent(QShowEvent *e);
     virtual bool acceptDrag(QDropEvent *e) const { return KURLDrag::canDecode(e); }
     virtual void viewportPaintEvent(QPaintEvent *pe);
     virtual void viewportResizeEvent(QResizeEvent *re);
 
+    virtual void insertItem(QListViewItem *item);
+    virtual void takeItem(QListViewItem *item);
+
+    virtual bool hasItem(const QString &file) const { return m_members.contains(file); }
+
     void addColumn(const QString &label);
 
     /**
@@ -395,56 +445,34 @@ protected:
      */
     void setupItem(PlaylistItem *item);
 
+    /**
+     * Forwards the call to the parent to enable or disable automatic deletion
+     * of tree view playlists.  Used by CollectionListItem.
+     */
+    void setCanDeletePlaylist(bool canDelete);
+
+    template <class ItemType, class SiblingType>
+    ItemType *createItem(SiblingType *sibling, ItemType *after = 0);
+
     /**
      * As a template this allows us to use the same code to initialize the items
      * in subclasses.  CollectionItemType should always be CollectionListItem and
      * ItemType should be a PlaylistItem subclass.
      */
     template <class CollectionItemType, class ItemType, class SiblingType>
-    void createItems(const QValueList<SiblingType *> &siblings);
+    void createItems(const QValueList<SiblingType *> &siblings, ItemType *after = 0);
 
-    /**
-     * Though it's somewhat obvious, this function will stat the file, so only use it when
-     * you're out of a performance critical loop.
-     */
-    static QString resolveSymLinks(const QFileInfo &file);
+protected slots:
+    void slotPopulateBackMenu() const;
+    void slotPlayFromBackMenu(int number) const;
 
 signals:
-    /**
-     * This is emitted when the playlist selection is changed.  This is used
-     * primarily to notify the TagEditor of the new data.
-     */
-    void signalSelectionChanged(const PlaylistItemList &selection);
 
     /**
      * This is connected to the PlaylistBox::Item to let it know when the
      * playlist's name has changed.
      */
-    void signalNameChanged(const QString &fileName);
-
-    /**
-     * This signal is emitted when items are added to or removed from the list.
-     *
-     * \see signalDataChanged()
-     * \see signalChanged()
-     */
-    void signalCountChanged(Playlist *);
-
-    /**
-     * This signal is connected to PlaylistItem::refreshed() in the PlaylistItem
-     * class.  It is emitted when a playlist item's data has been changed.
-     *
-     * \see signalCountChanged()
-     * \see signalChanged()
-     */
-    void signalDataChanged();
-
-    /**
-     * This is the union of signalDataChanged() and signalCountChanged().
-     * It is emitted with either quantity or value of the PlaylistItems are
-     * changed.
-     */
-    void signalChanged();
+    void signalNameChanged(const QString &name);
 
     /**
      * This signal is emitted just before a playlist item is removed from the
@@ -454,21 +482,11 @@ signals:
      */
     void signalAboutToRemove(PlaylistItem *item);
 
-    /**
-     * This is emitted when \a files are dropped on a specific playlist.
-     */
-    void signalFilesDropped(const QStringList &files, Playlist *, PlaylistItem *after);
+    void signalEnableDirWatch(bool enable);
 
-    /**
-     * Set the next item to be played in the current playlist.  This is used by
-     * the "Play Next" feature.
-     */
-    void signalSetNext(PlaylistItem *item);
+    void coverChanged();
 
-    /**
-     * Request creation of a playlist based on \a items.
-     */
-    void signalCreatePlaylist(const PlaylistItemList &items);
+    void signalPlaylistItemsDropped(Playlist *p);
 
 private:
     void setup();
@@ -483,9 +501,9 @@ private:
 
     /**
      * Writes \a text to \a item in \a column.  This is used by the inline tag
-     * editor.
+     * editor.  Returns false if the tag update failed.
      */
-    void editTag(PlaylistItem *item, const QString &text, int column);
+    bool editTag(PlaylistItem *item, const QString &text, int column);
 
     /**
      * Returns the index of the left most visible column in the playlist.
@@ -509,6 +527,18 @@ private:
      */
     void calculateColumnWeights();
 
+    void addFile(const QString &file, FileHandleList &files, bool importPlaylists,
+		 PlaylistItem **after);
+    void addFileHelper(FileHandleList &files, PlaylistItem **after,
+		       bool ignoreTimer = false);
+
+    void redisplaySearch() { setSearch(m_search); }
+
+    void refreshAlbums(const PlaylistItemList &items, const QImage &image = QImage());
+    void refreshAlbum(const QString &artist, const QString &album);
+
+    void updatePlaying() const;
+
     /**
      * This class is used internally to store settings that are shared by all
      * of the playlists, such as column order.  It is implemented as a singleton.
@@ -519,11 +549,7 @@ private slots:
 
     void slotUpdateColumnWidths();
 
-    /**
-     * This is just used to emit the selection as a list of PlaylistItems when
-     * the selection changes.
-     */
-    void slotEmitSelected() { emit signalSelectionChanged(selectedItems()); }
+    void slotAddToUpcoming();
 
     /**
      * Show the RMB menu.  Matches the signature for the signal 
@@ -562,7 +588,7 @@ private slots:
     /**
      * Prompts the user to create a new playlist with from the selected items.
      */
-    void slotCreateGroup() { emit signalCreatePlaylist(selectedItems()); }
+    void slotCreateGroup();
 
     /**
      * This slot is called when the user drags the slider in the listview header
@@ -578,23 +604,31 @@ private slots:
      */
     void slotInlineCompletionModeChanged(KGlobalSettings::Completion mode);
 
+    void slotPlayCurrent();
+
 private:
+    PlaylistCollection *m_collection;
+
     StringHash m_members;
 
     int m_currentColumn;
     int m_processed;
-
-    int m_rmbPasteID;
     int m_rmbEditID;
-
+    int m_rmbUpcomingID;
     int m_selectedCount;
 
     bool m_allowDuplicates;
     bool m_polished;
     bool m_applySharedSettings;
+    bool m_mousePressed;
 
     QValueList<int> m_weightDirty;
     bool m_disableColumnWidthUpdates;
+
+    mutable int m_time;
+    mutable QValueList<PlaylistItem::Pointer> m_addTime;
+    mutable QValueList<PlaylistItem::Pointer> m_subtractTime;
+
     /**
      * The average minimum widths of columns to be used in balancing calculations.
      */
@@ -603,9 +637,11 @@ private:
     bool m_widthsDirty;
 
     PlaylistItemList m_randomList;
-    PlaylistItemList m_history;
+    static PlaylistItemList m_history;
     PlaylistSearch m_search;
 
+    bool m_searchEnabled;
+
     PlaylistItem *m_lastSelected;
 
     /**
@@ -625,6 +661,7 @@ private:
     KPopupMenu *m_rmbMenu;
     KPopupMenu *m_headerMenu;
     KActionMenu *m_columnVisibleAction;
+    PlaylistToolTip *m_toolTip;
 
     /**
      * This is used to indicate if the list of visible items has changed (via a 
@@ -632,57 +669,48 @@ private:
      */
     static bool m_visibleChanged;
     static int m_leftColumn;
-    static PlaylistItem *m_playingItem;
+    static QMap<int, PlaylistItem *> m_backMenuItems;
+
+    bool m_blockDataChanged;
 };
 
-QDataStream &operator<<(QDataStream &s, Playlist &p);
+bool processEvents();
+
+QDataStream &operator<<(QDataStream &s, const Playlist &p);
 QDataStream &operator>>(QDataStream &s, Playlist &p);
 
 // template method implementations
 
 template <class ItemType, class CollectionItemType, class CollectionListType>
-ItemType *Playlist::createItem(const QFileInfo &file, const QString &absFilePath,
-			       QListViewItem *after, bool emitChanged)
+ItemType *Playlist::createItem(const FileHandle &file, QListViewItem *after,
+			       bool emitChanged)
 {
-    QString filePath;
-
-    if(absFilePath.isNull())
-	filePath = resolveSymLinks(file);
-    else
-	filePath = absFilePath;
-
-    CollectionItemType *item = CollectionListType::instance()->lookup(filePath);
+    CollectionItemType *item = CollectionListType::instance()->lookup(file.absFilePath());
 
     if(!item) {
-	item = new CollectionItemType(file, filePath);
+	item = new CollectionItemType(file);
 	setupItem(item);
 
 	// If a valid tag was not created, destroy the CollectionListItem.
+
 	if(!item->isValid()) {
 	    kdError(65432) << "Playlist::createItem() -- A valid tag was not created for \""
-			   << file.filePath() << "\"" << endl;
+			   << file.absFilePath() << "\"" << endl;
 	    delete item;
 	    return 0;
 	}
     }
 
-    if(item && !m_members.insert(filePath) || m_allowDuplicates) {
-	ItemType *i;
-	if(after)
-	    i = new ItemType(item, this, after);
-	else
-	    i = new ItemType(item, this);
+    if(item && !m_members.insert(file.absFilePath()) || m_allowDuplicates) {
 
+	ItemType *i = after ? new ItemType(item, this, after) : new ItemType(item, this);
 	setupItem(i);
 
         if(!m_randomList.isEmpty() && !m_visibleChanged)
             m_randomList.append(i);
 
-	emit signalCountChanged(this);
-	connect(item, SIGNAL(signalAboutToDelete()), i, SLOT(slotClear()));
-
 	if(emitChanged)
-	    emit signalCountChanged(this);
+	    dataChanged();
 
 	return i;
     }
@@ -690,28 +718,34 @@ ItemType *Playlist::createItem(const QFileInfo &file, const QString &absFilePath
 	return 0;
 }
 
+template <class ItemType, class SiblingType>
+ItemType *Playlist::createItem(SiblingType *sibling, ItemType *after)
+{
+    m_disableColumnWidthUpdates = true;
+    
+    if(!m_members.insert(sibling->file().absFilePath()) || m_allowDuplicates) {
+	after = new ItemType(sibling->collectionItem(), this, after);
+	setupItem(after);
+	if(!m_randomList.isEmpty() && !m_visibleChanged)
+	    m_randomList.append(after);
+    }
+
+    m_disableColumnWidthUpdates = false;
+
+    return after;
+}
+
 template <class CollectionItemType, class ItemType, class SiblingType>
-void Playlist::createItems(const QValueList<SiblingType *> &siblings)
+void Playlist::createItems(const QValueList<SiblingType *> &siblings, ItemType *after)
 {
     if(siblings.isEmpty())
 	return;
 
-    m_disableColumnWidthUpdates = true;
-    ItemType *newItem = 0;
-
     QValueListConstIterator<SiblingType *> it = siblings.begin();
-    for(; it != siblings.end(); ++it) {
-	if(!m_members.insert(resolveSymLinks((*it)->absFilePath())) || m_allowDuplicates) {
-	    newItem = new ItemType((*it)->collectionItem(), this, newItem);
-	    setupItem(newItem);
-	    if(!m_randomList.isEmpty() && !m_visibleChanged)
-		m_randomList.append(newItem);
-	    connect((*it)->collectionItem(), SIGNAL(signalAboutToDelete()), newItem, SLOT(slotClear()));
-	}
-    }
+    for(; it != siblings.end(); ++it)
+	after = createItem(*it, after);
 
-    emit signalCountChanged(this);
-    m_disableColumnWidthUpdates = false;
+    dataChanged();
     slotWeightDirty();
 }
 
