@@ -24,7 +24,6 @@
 
 class Playlist;
 class PlaylistItem;
-class DynamicPlaylist;
 class ViewMode;
 class PlaylistSearch;
 class SearchPlaylist;
@@ -67,6 +66,9 @@ public slots:
     void paste();
     void clear() {}
 
+    void slotFreezePlaylists();
+    void slotUnfreezePlaylists();
+
 protected:
     virtual void setupPlaylist(Playlist *playlist, const QString &iconName);
     virtual void setupPlaylist(Playlist *playlist, const QString &iconName, Item *parentItem);
@@ -92,6 +94,7 @@ private:
 
     void setupItem(Item *item);
     void performTreeViewSetup();
+    void setupUpcomingPlaylist();
     int viewModeIndex() const { return m_viewModeIndex; }
     ViewMode *viewMode() const { return m_viewModes[m_viewModeIndex]; }
 
@@ -104,15 +107,17 @@ private slots:
     void slotDoubleClicked();
     void slotShowContextMenu(QListViewItem *, const QPoint &point, int);
     void slotSetViewMode(int index);
-    void slotPlaylistDestroyed(Playlist*);
+    void slotTreeViewPlaylistDestroyed(Playlist*);
     void slotSavePlaylists();
+    void slotShowDropTarget();
+
+    void slotPlaylistItemsDropped(Playlist *p);
 
     void slotAddItem(const QString &tag, unsigned column);
     void slotRemoveItem(const QString &tag, unsigned column);
 
 private:
     KPopupMenu *m_contextMenu;
-    bool m_updatePlaylistStack;
     QPtrDict<Item> m_playlistDict;
     int m_viewModeIndex;
     QValueList<ViewMode *> m_viewModes;
@@ -121,7 +126,7 @@ private:
     bool m_doingMultiSelect;
     bool m_treeViewSetup;
     Item *m_dropItem;
-    DynamicPlaylist *m_dynamicPlaylist;
+    QTimer *m_showTimer;
 };
 
 
@@ -143,7 +148,7 @@ public:
     virtual ~Item();
     
 protected:
-    Item(PlaylistBox *listBox, const QString &icon, const QString &text, Playlist *l = 0);
+    Item(PlaylistBox *listBox, const QString &icon, const QString &text, Playlist *l = 0, Item *after = 0);
     Item(Item *parent, const QString &icon, const QString &text, Playlist *l = 0);
 
     Playlist *playlist() const { return m_playlist; }
