@@ -25,9 +25,13 @@
 #include <kdebug.h>
 
 #include <qvbox.h>
+#include <qtimer.h>
+#include <qcolor.h>
 #include <qpushbutton.h>
 #include <qtooltip.h>
+#include <qpainter.h>
 #include <qvaluevector.h>
+#include <qstylesheet.h>
 
 #include <netwm.h>
 
@@ -35,11 +39,49 @@
 #include "systemtray.h"
 #include "actioncollection.h"
 #include "playermanager.h"
+#include "collectionlist.h"
+#include "coverinfo.h"
 
 using namespace ActionCollection;
 
 static bool copyImage(QImage &dest, QImage &src, int x, int y);
 
+class FlickerFreeLabel : public QLabel
+{
+public:
+    FlickerFreeLabel(const QString &text, QWidget *parent, const char *name = 0) :
+        QLabel(text, parent, name)
+    {
+        m_textColor = paletteForegroundColor();
+        setBackgroundMode(NoBackground);
+    }
+
+    QColor textColor() const
+    {
+        return m_textColor;
+    }
+
+protected:
+    virtual void drawContents(QPainter *p)
+    {
+        // We want to intercept the drawContents call and draw on a pixmap
+        // instead of the window to keep flicker to an absolute minimum.
+        // Since Qt doesn't refresh the background, we need to do so
+        // ourselves.
+
+        QPixmap pix(size());
+        QPainter pixPainter(&pix);
+
+        pixPainter.fillRect(rect(), paletteBackgroundColor());
+        QLabel::drawContents(&pixPainter);
+
+        bitBlt(p->device(), QPoint(0, 0), &pix, rect(), CopyROP);
+    }
+
+    private:
+    QColor m_textColor;
+};
+
 class PassiveInfo : public KPassivePopup
 {
 public:
@@ -66,6 +108,10 @@ SystemTray::SystemTray(QWidget *parent, const char *name) : KSystemTray(parent,
                                                             m_popup(0)
 
 {
+    // This should be initialized to the number of labels that are used.
+
+    m_labels.reserve(3);
+
     m_appPix = loadIcon("juk_dock");
 
     m_playPix = createPixmap("player_play");
@@ -81,7 +127,8 @@ SystemTray::SystemTray(QWidget *parent, const char *name) : KSystemTray(parent,
     // Just create this here so that it show up in the DCOP interface and the key
     // bindings dialog.
 
-    new KAction(i18n("Redisplay Popup"), KShortcut(), this, SLOT(slotPlay()), actions(), "showPopup");
+    new KAction(i18n("Redisplay Popup"), KShortcut(), this,
+                SLOT(slotPlay()), actions(), "showPopup");
     
     KPopupMenu *cm = contextMenu();
 
@@ -132,6 +179,23 @@ void SystemTray::slotPlay()
     createPopup();
 }
 
+void SystemTray::slotTogglePopup()
+{
+    if(m_popup && m_popup->view()->isVisible())
+        m_popup->setTimeout(50);
+    else
+        slotPlay();
+}
+
+void  SystemTray::slotPopupLargeCover()
+{
+    if(!PlayerManager::instance()->playing())
+        return;
+
+    FileHandle playingFile = PlayerManager::instance()->playingFile();
+    playingFile.coverInfo()->popup();
+}
+
 void SystemTray::slotStop()
 {
     setPixmap(m_appPix);
@@ -141,6 +205,37 @@ void SystemTray::slotStop()
     m_popup = 0;
 }
 
+void SystemTray::slotClearLabels()
+{
+    for(unsigned i = 0; i < m_labels.capacity(); ++i)
+        m_labels[i] = 0;
+}
+
+void SystemTray::slotNextStep()
+{
+    static const int steps = 20;
+    QColor result, start, end;
+    int r, g, b;
+
+    ++m_step;
+    for(unsigned i = 0; i < m_labels.capacity() && m_labels[i]; ++i) {
+        start = m_labels[i]->paletteBackgroundColor();
+        result = end = m_labels[i]->textColor();
+
+        if(m_step < steps) {
+            r = (m_step * end.red() + (steps - m_step) * start.red()) / steps;
+            g = (m_step * end.green() + (steps - m_step) * start.green()) / steps;
+            b = (m_step * end.blue() + (steps - m_step) * start.blue()) / steps;
+            result = QColor(r, g, b);
+        }
+
+        m_labels[i]->setPaletteForegroundColor(result);
+    }
+
+    if(m_step < steps)
+        QTimer::singleShot(1500 / steps, this, SLOT(slotNextStep()));
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // private methods
 ////////////////////////////////////////////////////////////////////////////////
@@ -157,60 +252,60 @@ void SystemTray::createPopup()
         delete m_popup;
         m_popup = new PassiveInfo(this);
 
+        connect(m_popup, SIGNAL(destroyed()), SLOT(slotClearLabels()));
+        m_step = 0;
+
         QHBox *box = new QHBox(m_popup);
         box->setSpacing(15); // Add space between text and buttons
 
         // See where to put the buttons
 
         bool onLeft = buttonsToLeft();
-        QVBox *buttonBox;
         QVBox *infoBox;
 
         if(onLeft) {
 
             // They go to the left because JuK is on that side
 
-            buttonBox = new QVBox(box);
-
-            // Separator line
-
-            QFrame *line = new QFrame(box);
-            line->setFrameShape(QFrame::VLine);
+            createButtonBox(box);
+            addSeparatorLine(box);
 
             infoBox = new QVBox(box);
+
+            // Another line, and the cover, if there's a cover, and if
+            // it's selected to be shown
+
+            if(playingFile.coverInfo()->hasCover()) {
+                addSeparatorLine(box);
+                addCoverButton(box, playingFile.coverInfo()->pixmap(CoverInfo::Thumbnail));
+            }
         }
         else {
 
             // Buttons go on right because JuK is there
 
+            // Another line, and the cover, if there's a cover, and if
+            // it's selected to be shown
+
+            if(playingFile.coverInfo()->hasCover()) {
+                addCoverButton(box, playingFile.coverInfo()->pixmap(CoverInfo::Thumbnail));
+                addSeparatorLine(box);
+            }
+
             infoBox = new QVBox(box);
 
-            QFrame *line = new QFrame(box);
-            line->setFrameShape(QFrame::VLine);
-
-            buttonBox = new QVBox(box);
+            addSeparatorLine(box);
+            createButtonBox(box);
         }
 
         infoBox->setSpacing(3);
         infoBox->setMargin(3);
-        buttonBox->setSpacing(3);
         
-        // This should be initialized to the number of labels that are used.
-
-        QValueVector<QLabel *> labels(3, 0);
-        for(QValueVector<QLabel *>::Iterator it = labels.begin(); it != labels.end(); ++it) {
-            *it = new QLabel(" ", infoBox);
-            (*it)->setAlignment(AlignRight | AlignVCenter);
+        for(unsigned i = 0; i < m_labels.capacity(); ++i) {
+            m_labels[i] = new FlickerFreeLabel(" ", infoBox);
+            m_labels[i]->setAlignment(AlignRight | AlignVCenter);
         }
 
-        QPushButton *forwardButton = new QPushButton(m_forwardPix, 0, buttonBox, "popup_forward");
-        forwardButton->setFlat(true);
-        connect(forwardButton, SIGNAL(clicked()), action("forward"), SLOT(activate()));
-
-        QPushButton *backButton = new QPushButton(m_backPix, 0, buttonBox, "popup_back");
-        backButton->setFlat(true);
-        connect(backButton, SIGNAL(clicked()), action("back"), SLOT(activate()));
-
         // We don't want an autodelete popup.  There are times when it will need
         // to be hidden before the timeout.
 
@@ -222,18 +317,22 @@ void SystemTray::createPopup()
 
         int labelCount = 0;
 
-        labels[labelCount++]->setText(QString("<nobr><h2>%1</h2></nobr>").arg(playingInfo->title()));
+        QString title = QStyleSheet::escape(playingInfo->title());
+        m_labels[labelCount++]->setText(QString("<qt><nobr><h2>%1</h2></nobr><qt>").arg(title));
 
         if(!playingInfo->artist().isEmpty())
-            labels[labelCount++]->setText(playingInfo->artist());
+            m_labels[labelCount++]->setText(playingInfo->artist());
 
         if(!playingInfo->album().isEmpty()) {
+            QString album = QStyleSheet::escape(playingInfo->album());
             QString s = playingInfo->year() > 0
-                ? QString("%1 (%2)").arg(playingInfo->album()).arg(playingInfo->year())
-                : playingInfo->album();
-            labels[labelCount++]->setText(s);
+                ? QString("<qt><nobr>%1 (%2)</nobr></qt>").arg(album).arg(playingInfo->year())
+                : QString("<qt><nobr>%1</nobr></qt>").arg(album);
+            m_labels[labelCount++]->setText(s);
         }
 
+        slotNextStep();
+
         m_popup->setView(box);
         m_popup->show();
     }
@@ -271,6 +370,43 @@ QPixmap SystemTray::createPixmap(const QString &pixName)
     return bgPix;
 }
 
+void SystemTray::createButtonBox(QWidget *parent)
+{
+    QVBox *buttonBox = new QVBox(parent);
+    
+    buttonBox->setSpacing(3);
+
+    QPushButton *forwardButton = new QPushButton(m_forwardPix, 0, buttonBox, "popup_forward");
+    forwardButton->setFlat(true);
+    connect(forwardButton, SIGNAL(clicked()), action("forward"), SLOT(activate()));
+
+    QPushButton *backButton = new QPushButton(m_backPix, 0, buttonBox, "popup_back");
+    backButton->setFlat(true);
+    connect(backButton, SIGNAL(clicked()), action("back"), SLOT(activate()));
+}
+
+void SystemTray::addSeparatorLine(QWidget *parent)
+{
+    QFrame *line = new QFrame(parent);
+    line->setFrameShape(QFrame::VLine);
+
+    // Cover art takes up 80 pixels, make sure we take up at least 80 pixels
+    // even if we don't show the cover art for consistency.
+
+    line->setMinimumHeight(80);
+}
+
+void SystemTray::addCoverButton(QWidget *parent, const QPixmap &cover)
+{
+    QPushButton *coverButton = new QPushButton(parent);
+
+    coverButton->setPixmap(cover);
+    coverButton->setFixedSize(cover.size());
+    coverButton->setFlat(true);
+
+    connect(coverButton, SIGNAL(clicked()), this, SLOT(slotPopupLargeCover()));
+}
+
 void SystemTray::setToolTip(const QString &tip)
 {
     QToolTip::remove(this);
@@ -376,6 +512,7 @@ static bool copyImage(QImage &dest, QImage &src, int x, int y)
     return true;
 }
 
+
 #include "systemtray.moc"
 
-// vim: ts=8
+// vim: et sw=4 ts=8
