@@ -1,9 +1,9 @@
 /***************************************************************************
-                          systray.cpp  -  description
-                             -------------------
-
-    copyright            : (C) 2002 by Daniel Molkentin,
+    copyright            : (C) 2002 by Daniel Molkentin
     email                : molkentin@kde.org
+
+    copyright            : (C) 2002 - 2004 by Scott Wheeler
+    email                : wheeler@kde.org
  ***************************************************************************/
 
 /***************************************************************************
@@ -20,32 +20,42 @@
 
 #include <ksystemtray.h>
 
+#include <qvaluevector.h>
+
+class FlickerFreeLabel;
 class QTimer;
 class KPassivePopup;
-class KMainWindow;
-class KAction;
-class KToggleAction;
 
 class SystemTray : public KSystemTray
 {
     Q_OBJECT
 
 public:
-    SystemTray(KMainWindow *parent = 0, const char *name = 0);
+    SystemTray(QWidget *parent = 0, const char *name = 0);
     virtual ~SystemTray();
 
-public slots:
-    void slotNewSong(const QString &songName);
-    void slotPlay() { setPixmap(m_playPix); }
-    void slotPause() { setPixmap(m_pausePix); }
-    void slotStop();
-
 private:
-    virtual void wheelEvent( QWheelEvent *e);
-    void createPopup(const QString &songName, bool addButtons = true);
+    virtual void wheelEvent(QWheelEvent *e);
+    void createPopup();
     void setToolTip(const QString &tip = QString::null);
+    void mousePressEvent(QMouseEvent *e);
+    bool buttonsToLeft() const;
     QPixmap createPixmap(const QString &pixName);
 
+    void createButtonBox(QWidget *parent);
+    void addSeparatorLine(QWidget *parent);
+    void addCoverButton(QWidget *parent, const QPixmap &cover);
+
+private slots:
+    void slotPlay();
+    void slotTogglePopup();
+    void slotPause() { setPixmap(m_pausePix); }
+    void slotStop();
+    void slotClearLabels();
+    void slotNextStep();
+    void slotPopupLargeCover();
+
+private:
     QPixmap m_playPix;
     QPixmap m_pausePix;
     QPixmap m_currentPix;
@@ -55,14 +65,8 @@ private:
 
     KPassivePopup *m_popup;
     QLabel *m_currentLabel;
-
-    KActionCollection *m_actionCollection;
-    KAction *m_playAction;
-    KAction *m_pauseAction;
-    KAction *m_stopAction;
-    KAction *m_backAction;
-    KAction *m_forwardAction;
-    KToggleAction *m_togglePopupsAction;
+    QValueVector<FlickerFreeLabel *> m_labels;
+    int m_step;
 };
 
 #endif // SYSTEMTRAY_H
