@@ -1,8 +1,6 @@
 /***************************************************************************
-                          tag.cpp  -  description
-                             -------------------
     begin                : Sun Feb 17 2002
-    copyright            : (C) 2002 by Scott Wheeler
+    copyright            : (C) 2002 - 2004 by Scott Wheeler
     email                : wheeler@kde.org
 ***************************************************************************/
 
@@ -23,12 +21,21 @@
 #include <taglib/tag.h>
 #include <taglib/mpegfile.h>
 #include <taglib/vorbisfile.h>
-#if defined TAGLIB_MINOR_VERSION && TAGLIB_MINOR_VERSION >= 96
 #include <taglib/flacfile.h>
-#endif
 #include <taglib/xiphcomment.h>
 #include <taglib/id3v2framefactory.h>
 
+#if (TAGLIB_MAJOR_VERSION > 1) || \
+      ((TAGLIB_MAJOR_VERSION == 1) && (TAGLIB_MINOR_VERSION >= 2))
+#include <taglib/oggflacfile.h>
+#define TAGLIB_1_2
+#endif
+#if (TAGLIB_MAJOR_VERSION > 1) || \
+    ((TAGLIB_MAJOR_VERSION == 1) && (TAGLIB_MINOR_VERSION >= 3))
+#include <taglib/mpcfile.h>
+#define TAGLIB_1_3
+#endif
+
 #include "cache.h"
 #include "tag.h"
 #include "mediafiles.h"
@@ -38,74 +45,76 @@
 // public members
 ////////////////////////////////////////////////////////////////////////////////
 
-Tag *Tag::createTag(const QString &fileName, bool ignoreCache)
+
+Tag::Tag(const QString &fileName) :
+    m_fileName(fileName),
+    m_track(0),
+    m_year(0),
+    m_seconds(0),
+    m_bitrate(0),
+    m_isValid(false)
 {
-    Tag *cachedItem = 0;
-
-    if(!ignoreCache)
-        cachedItem = Cache::instance()->find(fileName);
-
-    if(cachedItem)
-        return cachedItem;
-
     if(MediaFiles::isMP3(fileName)) {
         TagLib::MPEG::File file(QFile::encodeName(fileName).data());
-        if(!file.isValid())
-            return 0;
-        return new Tag(fileName, &file);
+        if(file.isValid())
+            setup(&file);
     }
 
-    if(MediaFiles::isOgg(fileName)) {
-        TagLib::Vorbis::File file(QFile::encodeName(fileName).data());
-        if(!file.isValid())
-            return 0;
-        return new Tag(fileName, &file);
-    }
-
-#if defined TAGLIB_MINOR_VERSION && TAGLIB_MINOR_VERSION >= 96
-    if(MediaFiles::isFLAC(fileName)) {
+    else if(MediaFiles::isFLAC(fileName)) {
         TagLib::FLAC::File file(QFile::encodeName(fileName).data());
-        if(!file.isOpen())
-            return 0;
-        return new Tag(fileName, &file);
+        if(file.isOpen())
+            setup(&file);
+    }
+#ifdef TAGLIB_1_3
+    else if(MediaFiles::isMPC(fileName)) {
+        kdDebug(65432) << "Trying to resolve Musepack file" << endl;
+        TagLib::MPC::File file(QFile::encodeName(fileName).data());
+        if(file.isOpen())
+            setup(&file);
     }
 #endif
+#ifdef TAGLIB_1_2
+    else if(MediaFiles::isOggFLAC(fileName)) {
+        TagLib::Ogg::FLAC::File file(QFile::encodeName(fileName).data());
+        if(file.isOpen())
+            setup(&file);
+    }
+#endif
+    else if(MediaFiles::isVorbis(fileName)) {
+        TagLib::Vorbis::File file(QFile::encodeName(fileName).data());
+        if(file.isValid())
+            setup(&file);
+    }
 
-    kdError(65432) << "Couldn't resolve the mime type of \"" <<
-        fileName << "\" -- this shouldn't happen." << endl;
-
-    return 0;
+    else {
+        kdError(65432) << "Couldn't resolve the mime type of \"" <<
+            fileName << "\" -- this shouldn't happen." << endl;
+    }
 }
 
-Tag::~Tag()
-{
-    Cache::instance()->remove(m_fileName);
-}
-
-void Tag::save()
+bool Tag::save()
 {
+    bool result;
     TagLib::ID3v2::FrameFactory::instance()->setDefaultTextEncoding(TagLib::String::UTF8);
 
-    if(!m_info.isWritable())
-        return;
-
     TagLib::File *file = 0;
 
     if(MediaFiles::isMP3(m_fileName))
         file = new TagLib::MPEG::File(QFile::encodeName(m_fileName).data());
-#if defined TAGLIB_MINOR_VERSION && TAGLIB_MINOR_VERSION >= 95
-    else if(MediaFiles::isOgg(m_fileName))
-        file = new TagLib::Vorbis::File(QFile::encodeName(m_fileName).data());
-#else
-#ifdef _GNUC
-#warning "Your TagLib is too old for saving Vorbis files.  It is being disabled for now."
-#endif
-#endif
-#if defined TAGLIB_MINOR_VERSION && TAGLIB_MINOR_VERSION >= 96
     else if(MediaFiles::isFLAC(m_fileName))
         file = new TagLib::FLAC::File(QFile::encodeName(m_fileName).data());
+#ifdef TAGLIB_1_3
+    else if(MediaFiles::isMPC(m_fileName))
+        file = new TagLib::MPC::File(QFile::encodeName(m_fileName).data());
 #endif
-    if(file && file->isValid() && file->tag()) {
+#ifdef TAGLIB_1_2
+    else if(MediaFiles::isOggFLAC(m_fileName))
+        file = new TagLib::Ogg::FLAC::File(QFile::encodeName(m_fileName).data());
+#endif
+    else if(MediaFiles::isVorbis(m_fileName))
+        file = new TagLib::Vorbis::File(QFile::encodeName(m_fileName).data());
+
+    if(file && file->isValid() && file->tag() && !file->readOnly()) {
         file->tag()->setTitle(QStringToTString(m_title));
         file->tag()->setArtist(QStringToTString(m_artist));
         file->tag()->setAlbum(QStringToTString(m_album));
@@ -113,30 +122,22 @@ void Tag::save()
         file->tag()->setComment(QStringToTString(m_comment));
         file->tag()->setTrack(m_track);
         file->tag()->setYear(m_year);
-
+#ifdef TAGLIB_1_2
+        result = file->save();
+#else
         file->save();
-    } else
+        result = true;
+#endif
+    }
+    else {
         kdError(65432) << "Couldn't save file." << endl;
-
+        result = false;
+    }
 
     delete file;
+    return result;
 }
 
-bool Tag::current() const
-{
-    return(m_modificationTime.isValid() &&
-           lastModified().isValid() &&
-           m_modificationTime >= Tag::lastModified());
-}
-
-QDateTime Tag::lastModified() const
-{
-    if(m_lastModified.isNull())
-        m_lastModified = m_info.lastModified();
-    return m_lastModified;
-}
-
-
 CacheDataStream &Tag::read(CacheDataStream &s)
 {
     switch(s.cacheVersion()) {
@@ -155,8 +156,7 @@ CacheDataStream &Tag::read(CacheDataStream &s)
           >> m_comment
           >> bitrate
           >> m_lengthString
-          >> seconds
-          >> m_modificationTime;
+          >> seconds;
 
         m_track = track;
         m_year = year;
@@ -183,8 +183,7 @@ CacheDataStream &Tag::read(CacheDataStream &s)
           >> bitrateString
           >> m_lengthString
           >> m_seconds
-          >> dummyString
-          >> m_modificationTime;
+          >> dummyString;
 
         bool ok;
         m_bitrate = bitrateString.toInt(&ok);
@@ -211,20 +210,18 @@ CacheDataStream &Tag::read(CacheDataStream &s)
 // private methods
 ////////////////////////////////////////////////////////////////////////////////
 
-Tag::Tag(const QString &file) :
-    m_info(file),
-    m_fileName(file),
+Tag::Tag(const QString &fileName, bool) :
+    m_fileName(fileName),
     m_track(0),
     m_year(0),
     m_seconds(0),
-    m_bitrate(0)
+    m_bitrate(0),
+    m_isValid(true)
 {
-    Cache::instance()->insert(file, this);
+
 }
 
-Tag::Tag(const QString &fileName, TagLib::File *file) :
-    m_info(fileName),
-    m_fileName(fileName)
+void Tag::setup(TagLib::File *file)
 {
     m_title   = TStringToQString(file->tag()->title()).stripWhiteSpace();
     m_artist  = TStringToQString(file->tag()->artist()).stripWhiteSpace();
@@ -243,10 +240,13 @@ Tag::Tag(const QString &fileName, TagLib::File *file) :
 
     m_lengthString = QString::number(minutes) + (seconds >= 10 ? ":" : ":0") + QString::number(seconds);
 
-    if(m_title.isEmpty())
-        m_title = m_info.baseName(true);
+    if(m_title.isEmpty()) {
+        int i = m_fileName.findRev('/');
+        int j = m_fileName.findRev('.');
+        m_title = i > 0 ? m_fileName.mid(i + 1, j - i - 1) : m_fileName;
+    }
 
-    Cache::instance()->insert(fileName, this);
+    m_isValid = true;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -264,8 +264,7 @@ QDataStream &operator<<(QDataStream &s, const Tag &t)
       << t.comment()
       << Q_INT32(t.bitrate())
       << t.lengthString()
-      << Q_INT32(t.seconds())
-      << t.lastModified();
+      << Q_INT32(t.seconds());
 
     return s;
 }
