@@ -37,10 +37,11 @@ Kaboodle::KaboodleApp::KaboodleApp()
 	if(args->count() > 0)
 	{
 		KURL url = args->url(args->count() - 1);
-		if(!url.isMalformed()) openURL = url;
+		if(url.isValid()) openURL = url;
 	}
 
 	ui = new UserInterface(0L, openURL);
+	setMainWidget( ui );
 }
 
 Kaboodle::KaboodleApp::~KaboodleApp()
