@@ -129,7 +129,7 @@ void Encoder::removeJob(int id ) {
  * @param job the job to encode.
  */
 void Encoder::encodeWav(Job *job ) {
-	emit(addJob(job, i18n("Encoding (%1): %2 - %3").arg(prefs->extension()).arg(job->group).arg(job->track_title)));
+	emit(addJob(job, i18n("Encoding (%1): %2 - %3").arg(prefs->extension()).arg(job->track_artist).arg(job->track_title)));
 	pendingJobs.append(job);
 	tendToNewJobs();
 }
@@ -162,7 +162,7 @@ void Encoder::tendToNewJobs() {
 	{
 		QMap <QString,QString> map;
 		map.insert("extension", prefs->extension());
-		job->replaceSpecialChars(desiredFile, false, map);
+		desiredFile = job->replaceSpecialChars(desiredFile, false, map);
 	}
 	desiredFile.replace( QRegExp("~"), QDir::homeDirPath() );
 
@@ -200,7 +200,7 @@ void Encoder::tendToNewJobs() {
 		map.insert("extension", prefs->extension());
 		map.insert("f", job->location);
 		map.insert("o", desiredFile);
-		job->replaceSpecialChars(command, true, map);
+		command = job->replaceSpecialChars(command, true, map);
 	}
 
 	updateProgress(job->id, 1);
@@ -235,7 +235,7 @@ void Encoder::receivedThreadOutput(KProcess *process, char *buffer, int length )
 	// Make sure we have a job to send an update too.
 	Job *job = jobs[(KShellProcess*)process];
 	if ( !job ) {
-		qDebug("Encoder::receivedThreadOutput Job doesn't exists. Line: %d", __LINE__);
+		kdDebug(60002) << "Encoder::receivedThreadOutput Job doesn't exists. Line: " <<  __LINE__ << endl;
 		return;
 	}
 
@@ -245,7 +245,7 @@ void Encoder::receivedThreadOutput(KProcess *process, char *buffer, int length )
 	// Make sure the output string has a % symble in it.
 	QString output = QString(buffer).mid(0,length);
 	if ( output.find('%') == -1 && reportCount < 5 ) {
-		qDebug("No \'%%\' in output. Report as bug w/encoder options if progressbar doesn't fill.");
+		kdDebug(60002) << "No \'%%\' in output. Report as bug w/encoder options if progressbar doesn't fill." << endl;
 		reportCount++;
 		return;
 	}
@@ -260,8 +260,8 @@ void Encoder::receivedThreadOutput(KProcess *process, char *buffer, int length )
 	}
 	// If it was just some random output that couldn't be converted then don't report the error.
 	else
-		if ( conversionSuccessfull)
-			qDebug("Percent done:\"%d\" is not >= 0 && < 100.", percent);
+		if ( conversionSuccessfull )
+			kdWarning("Percent done:\"%d\" is not >= 0 && < 100.", percent);
 }
 
 /**
