@@ -276,7 +276,7 @@ void JobQueImp::appendToPlaylist(Job* job, const QString &extension){
 	QString desiredFile = Prefs::playlistFileFormat();
 	QMap <QString,QString> map;
 	map.insert("extension", extension);
-	job->replaceSpecialChars(desiredFile, false, map);
+	desiredFile = job->replaceSpecialChars(desiredFile, false, map);
 
 	desiredFile.replace( QRegExp("~"), QDir::homeDirPath() );
 	// If the user wants anything regexp replaced do it now...
@@ -320,7 +320,7 @@ void QueListViewItem::paintCell (QPainter * p,const QColorGroup &cg,int column,
 
 	p->setPen(cg.base());
 	p->drawRect(0,0,width,height());
-	if(this->isSelected())
+	if(isSelected())
 		p->fillRect(1,1,width-2,height()-2,cg.highlight());
 	else
 		p->fillRect(1,1,width-2,height()-2,cg.base());
@@ -331,7 +331,7 @@ void QueListViewItem::paintCell (QPainter * p,const QColorGroup &cg,int column,
 
 	// show the text
 	p->setPen(cg.text());
-	if(this->isSelected())
+	if(isSelected())
 		p->setPen(cg.highlightedText());
 	if(percentDone != -1)
 	p->drawText(0,0,width-1,height()-1,AlignCenter,QString().setNum((int)percentDone) + "%");
@@ -342,7 +342,7 @@ void QueListViewItem::paintCell (QPainter * p,const QColorGroup &cg,int column,
 /**
  * Header for built in treelist item so we can have a progress bar in them.
  */
-QueListViewItem::QueListViewItem(QListView *parent, QString id, QString p , QString name, QString d, QString e) : QListViewItem(parent, id, p, name,d,e), percentDone(0), progressing(false) {
+QueListViewItem::QueListViewItem(QListView *parent, const QString id, const QString p , const QString name, const QString d, const QString e) : QListViewItem(parent, id, p, name,d,e), percentDone(0), progressing(false) {
 }
 
 #include "jobqueimp.moc"
