@@ -49,12 +49,13 @@ public:
   virtual const char * fileType() const { return "ogg"; };
   virtual const char * mimeType() const;
 
-  virtual void fillSongInfo(QString trackName,
-		            QString cdArtist,
-			    QString cdTitle,
-			    QString cdCategory,
+  virtual void fillSongInfo( const QString &trackName,
+	        const QString &cdArtist,
+			    const QString &cdTitle,
+			    const QString &cdCategory,
 			    int trackNumber,
-			    int cdYear);
+			    int cdYear,
+					const QString &comment );
   
   virtual long readInit(long size);
   virtual long read(int16_t * buf, int frames);
