@@ -57,6 +57,7 @@ class KMixDockWidget : public KSystemTray  {
 
  protected:
    void createMasterVolWidget();
+   void createActions();
    void mousePressEvent(QMouseEvent *);
    void mouseReleaseEvent(QMouseEvent *);
    void wheelEvent(QWheelEvent *);
@@ -69,6 +70,10 @@ class KMixDockWidget : public KSystemTray  {
    int  _oldToolTipValue;
    char _oldPixmapType;
    bool _volumePopup;
+ private slots:
+   void dockMute();
+   void selectMaster();
+   void handleNewMaster(int soundcard_id, int channel_id);
 };
 
 #endif
