@@ -42,6 +42,8 @@
 #include "viewinput.h"
 #include "viewoutput.h"
 #include "viewswitches.h"
+// KMix experimental
+//#include "viewgrid.h"
 #include "viewsurround.h"
 
 
@@ -53,23 +55,17 @@
    (c) A balancing slider
    (d) A label containg the mixer name
 */
-KMixerWidget::KMixerWidget( int _id, Mixer *mixer, const QString &mixerName, int mixerNum,
+KMixerWidget::KMixerWidget( int _id, Mixer *mixer, const QString &mixerName,
                             MixDevice::DeviceCategory categoryMask,
                             QWidget * parent, const char * name, ViewBase::ViewFlags vflags )
    : QWidget( parent, name ), _mixer(mixer), m_balanceSlider(0),
      m_topLayout(0),
-     //m_name( mixerName ), m_mixerName( mixerName ),
-     m_mixerNum( mixerNum ), m_id( _id ),
+     m_id( _id ),
      _iconsEnabled( true ), _labelsEnabled( false ), _ticksEnabled( false )
 
 {
     m_categoryMask = categoryMask;
 
-   _oWidget = 0;
-   _iWidget = 0;
-   _swWidget = 0;
-   // Create mixer device widgets
-
    if ( _mixer )
    {
       createLayout(vflags);
@@ -81,7 +77,7 @@ KMixerWidget::KMixerWidget( int _id, Mixer *mixer, const QString &mixerName, int
        QBoxLayout *layout = new QHBoxLayout( this );
        QString s = i18n("Invalid mixer");
        if ( !mixerName.isEmpty() )
-	   s += " \"" + mixerName + "\"";
+	   s.append(" \"").append(mixerName).append("\"");
        QLabel *errorLabel = new QLabel( s, this );
        errorLabel->setAlignment( QLabel::AlignCenter | QLabel::WordBreak );
        layout->addWidget( errorLabel );
@@ -112,32 +108,24 @@ void KMixerWidget::createLayout(ViewBase::ViewFlags vflags)
     m_ioTab = new KTabWidget( this, "ioTab" );
     m_topLayout->add( m_ioTab );
 
-    // *** Create Views *********************************************************************
-    _oWidget  = new ViewOutput  ( m_ioTab, "OutputTab" , _mixer, vflags );
-    _iWidget  = new ViewInput   ( m_ioTab, "InputTab"  , _mixer, vflags );
-    _swWidget = new ViewSwitches( m_ioTab, "SwitchTab" , _mixer, vflags );
-    if ( vflags & ViewBase::Experimental_SurroundView )
-	_surroundWidget = new ViewSurround( m_ioTab, "SurroundTab", _mixer, vflags );
 
-    // *** Create device widgets ************************************************************
-    _oWidget ->createDeviceWidgets();
-    _iWidget ->createDeviceWidgets();
-    _swWidget->createDeviceWidgets();
-    if ( vflags & ViewBase::Experimental_SurroundView )
-	_surroundWidget->createDeviceWidgets();
-
-    // *** Add Views to Tab *****************************************************************
-    m_ioTab->addTab( _oWidget , i18n("Output") );
-    m_ioTab->addTab( _iWidget , i18n("Input" ) );
-    if ( _swWidget->count() > 0 ) { // Add switches Tab only, if there are Switches
-	m_ioTab->addTab( _swWidget, i18n("Switches" ) );
-    }
-    else {
-	delete _swWidget;
-	_swWidget = 0;
-    }
-    if ( vflags & ViewBase::Experimental_SurroundView )
-	m_ioTab->addTab(_surroundWidget, i18n("Surround" ) );
+     /*******************************************************************
+      *  Now the main GUI is created.
+      * 1) Select a (GUI) profile,  which defines  which controls to show on which Tab
+      * 2a) Create the Tab's and the corresponding Views
+      * 2b) Create device widgets
+      * 2c) Add Views to Tab
+      ********************************************************************/
+      //KMixGUIProfile* prof = MixerToolbox::selectProfile(_mixer);
+      
+      
+	possiblyAddView(new ViewOutput  ( m_ioTab, "Output" , _mixer, vflags ) );
+	possiblyAddView(new ViewInput( m_ioTab, "Input"  , _mixer, vflags ) );
+	possiblyAddView(new ViewSwitches( m_ioTab, "Switches" , _mixer, vflags ) );
+	if ( vflags & ViewBase::Experimental_SurroundView )
+		possiblyAddView( new ViewSurround( m_ioTab, "Surround", _mixer, vflags ) );
+//    if ( vflags & ViewBase::Experimental_GridView )
+//		possiblyAddView( new ViewGrid( m_ioTab, "Grid", _mixer, vflags ) );
 
 
     // *** Lower part: Slider and Mixer Name ************************************************
@@ -162,112 +150,88 @@ void KMixerWidget::createLayout(ViewBase::ViewFlags vflags)
     QToolTip::add( m_balanceSlider, i18n("Left/Right balancing") );
 
     // --- "MenuBar" toggling from the various View's ---
-    connect( _oWidget, SIGNAL(toggleMenuBar()), parentWidget(), SLOT(toggleMenuBar()) );
-    connect( _iWidget, SIGNAL(toggleMenuBar()), parentWidget(), SLOT(toggleMenuBar()) );
-    if (_swWidget != 0 )
-	connect( _swWidget, SIGNAL(toggleMenuBar()), parentWidget(), SLOT(toggleMenuBar()) );
-    if ( vflags & ViewBase::Experimental_SurroundView )
-	connect( _surroundWidget, SIGNAL(toggleMenuBar()), parentWidget(), SLOT(toggleMenuBar()) );
+
 
 
     show();
     //    kdDebug(67100) << "KMixerWidget::createLayout(): EXIT\n";
 }
 
+void KMixerWidget::possiblyAddView(ViewBase* vbase)
+{
+	if ( vbase->count() == 0 )
+		delete vbase;
+	else {
+		_views.push_back(vbase);
+		vbase ->createDeviceWidgets();
+		m_ioTab->addTab( vbase , i18n(vbase->name()) );
+		connect( vbase, SIGNAL(toggleMenuBar()), parentWidget(), SLOT(toggleMenuBar()) );
+	}
+}
 
 void KMixerWidget::setIcons( bool on )
 {
-    for (int i=0; i<=2; i++) {
-	ViewBase* m_mixerWidget;
-	switch (i) {
-	case 0: m_mixerWidget =  _oWidget; break;
-	case 1: m_mixerWidget =  _iWidget; break;
-	case 2: if ( _swWidget == 0) continue; else m_mixerWidget = _swWidget; break;
-	default: kdError(67100) << "KMixerWidget::setIcons(): wrong _mixerWidget " << i << "\n";; continue;
-	}
-
-	KMixToolBox::setIcons(m_mixerWidget->_mdws, on);
-    }
+	for ( std::vector<ViewBase*>::iterator it = _views.begin(); it != _views.end();  it++) {
+		ViewBase* mixerWidget = *it;
+		KMixToolBox::setIcons(mixerWidget->_mdws, on);
+    } // for all tabs
 }
 
 void KMixerWidget::setLabels( bool on )
 {
     if ( _labelsEnabled!=on ) {
-	// value was changed
-	_labelsEnabled = on;
-	for (int i=0; i<=2; i++) {
-	    ViewBase* m_mixerWidget;
-	    switch (i) {
-	    case 0: m_mixerWidget =  _oWidget; break;
-	    case 1: m_mixerWidget =  _iWidget; break;
-	    case 2: if ( _swWidget == 0) continue; else m_mixerWidget = _swWidget; break;
-	    default: kdError(67100) << "KMixerWidget::setLabels(): wrong _mixerWidget " << i << "\n"; continue;
-	    }
-
-	    KMixToolBox::setLabels(m_mixerWidget->_mdws, on);
-	}
+		// value was changed
+		_labelsEnabled = on;
+		for ( std::vector<ViewBase*>::iterator it = _views.begin(); it != _views.end();  it++) {
+			ViewBase* mixerWidget = *it;
+			KMixToolBox::setLabels(mixerWidget->_mdws, on);
+	    } // for all tabs
     }
 }
 
 void KMixerWidget::setTicks( bool on )
 {
     if ( _ticksEnabled!=on ) {
-	// value was changed
-	_ticksEnabled = on;
-	for (int i=0; i<=2; i++) {
-	    ViewBase* m_mixerWidget;
-	    switch (i) {
-	    case 0: m_mixerWidget =  _oWidget; break;
-	    case 1: m_mixerWidget =  _iWidget; break;
-	    case 2: if ( _swWidget == 0) continue; else m_mixerWidget = _swWidget; break;
-	    default: kdError(67100) << "KMixerWidget::setTicks(): wrong _mixerWidget " << i << "\n"; continue;
-	    }
-
-	    KMixToolBox::setTicks(m_mixerWidget->_mdws, on);
-	}
+		// value was changed
+		_ticksEnabled = on;
+		for ( std::vector<ViewBase*>::iterator it = _views.begin(); it != _views.end();  it++) {
+			ViewBase* mixerWidget = *it;
+		    KMixToolBox::setTicks(mixerWidget->_mdws, on);
+		} // for all tabs
     }
 }
 
 
-void
-KMixerWidget::loadConfig( KConfig *config, const QString &grp )
+/**
+ *  @todo : Is the view list already filled, when loadConfig() is called?
+ */
+void KMixerWidget::loadConfig( KConfig *config, const QString &grp )
 {
-  for (int i=0; i<=2; i++) {
-      ViewBase* m_mixerWidget;
-      QString viewPrefix = "View.";
-      switch (i) {
-      case 0: m_mixerWidget =  _oWidget; viewPrefix += "Output"   ; break;
-      case 1: m_mixerWidget =  _iWidget; viewPrefix += "Input"    ; break;
-      case 2: if ( _swWidget == 0) continue; else m_mixerWidget = _swWidget; viewPrefix += "Switches" ; break;
-      default: kdError(67100) << "KMixerWidget::loadConfig(): wrong _mixerWidget " << i << "\n"; continue;
-      }
 
-      KMixToolBox::loadConfig(m_mixerWidget->_mdws, config, grp, viewPrefix );
-      m_mixerWidget->configurationUpdate();
-  } // for the 3 tabs
+	for ( std::vector<ViewBase*>::iterator it = _views.begin(); it != _views.end();  it++) {
+		ViewBase* mixerWidget = *it;
+		QString viewPrefix = "View.";
+		viewPrefix += mixerWidget->name();
+		KMixToolBox::loadConfig(mixerWidget->_mdws, config, grp, viewPrefix );
+		mixerWidget->configurationUpdate();
+	} // for all tabs
 }
 
 
 
 void KMixerWidget::saveConfig( KConfig *config, const QString &grp )
 {
-    config->setGroup( grp );
-    // Write mixer name. It cannot be changed in the Mixer instance,
-    // it is only saved for diagnostical purposes (analyzing the config file).
-    config->writeEntry("Mixer_Name_Key", _mixer->mixerName());
-
-    for (int i=0; i<=2; i++) {
-	ViewBase* m_mixerWidget;
-	QString viewPrefix = "View.";
-	switch (i) {
-	case 0: m_mixerWidget =  _oWidget; viewPrefix += "Output"   ; break;
-	case 1: m_mixerWidget =  _iWidget; viewPrefix += "Input"    ; break;
-	case 2: if ( _swWidget == 0) continue; else m_mixerWidget = _swWidget; viewPrefix += "Switches" ; break;
-	default: kdError(67100) << "KMixerWidget::saveConfig(): wrong _mixerWidget " << i << "\n"; continue;
-	}
-
-	KMixToolBox::saveConfig(m_mixerWidget->_mdws, config, grp, viewPrefix );
-    } // for the 3 tabs
+	config->setGroup( grp );
+	// Write mixer name. It cannot be changed in the Mixer instance,
+	// it is only saved for diagnostical purposes (analyzing the config file).
+	config->writeEntry("Mixer_Name_Key", _mixer->mixerName());
+	
+	for ( std::vector<ViewBase*>::iterator it = _views.begin(); it != _views.end();  it++) {
+		ViewBase* mixerWidget = *it;
+		QString viewPrefix = "View.";
+		viewPrefix += mixerWidget->name();
+		KMixToolBox::saveConfig(mixerWidget->_mdws, config, grp, viewPrefix );
+	} // for all tabs
 }
 
 
