@@ -113,57 +113,45 @@ const uint_t MixerSunPortMasks[] =
 // FUNCTION    : SUN_getMixer
 // DESCRIPTION : Creates and returns a new mixer object.
 //======================================================================
-Mixer* SUN_getMixer( int devnum, int SetNum )
+Mixer_Backend* SUN_getMixer( int devnum )
 {
-   Mixer *l_mixer;
-   l_mixer = new Mixer_SUN( devnum, SetNum );
+   Mixer_Backend *l_mixer;
+   l_mixer = new Mixer_SUN( devnum );
    return l_mixer;
 }
 
-//======================================================================
-// FUNCTION    : SUN_getMixerSet
-// DESCRIPTION : Creates and returns a new mixer object.
-//======================================================================
-/*
-Mixer* SUN_getMixerSet( MixSet set, int device, int card )
-{
-   Mixer *l_mixer;
-   l_mixer = new Mixer_SUN( device, card );
-   return l_mixer;
-}
-*/
 
 //======================================================================
 // FUNCTION    : Mixer::Mixer
 // DESCRIPTION : Class constructor.
 //======================================================================
-Mixer_SUN::Mixer_SUN(int devnum, int card) : Mixer(devnum, card)
+Mixer_SUN::Mixer_SUN(int devnum) : Mixer_Backend(devnum)
 {
    if ( devnum == -1 )
       m_devnum = 0;
-   if ( card == -1 )
-      m_cardnum = 0;
 }
 
 //======================================================================
-// FUNCTION    : Mixer::openMixer
+// FUNCTION    : Mixer::Mixer
+// DESCRIPTION : Class destructor.
+//======================================================================
+Mixer_SUN::~Mixer_SUN(int devnum) : Mixer_Backend(devnum)
+{
+   close();
+}
+
+//======================================================================
+// FUNCTION    : Mixer::open
 // DESCRIPTION : Initialize the mixer and open the hardware driver.
 //======================================================================
-int Mixer_SUN::openMixer()
+int Mixer_SUN::open()
 {
    //
-   // We don't support multiple cards or devices
+   // We don't support multiple devices
    //
-   if ( m_cardnum != 0 )
-      return Mixer::ERR_OPEN;
    if ( m_devnum !=0 )
       return Mixer::ERR_OPEN;
 
-   //
-   // Release mixer before (re-)opening
-   //
-   release();
-
    //
    // Open the mixer hardware driver
    //
@@ -202,7 +190,7 @@ int Mixer_SUN::openMixer()
         {
            MixDevice* md = m_mixDevices.at( idx );
            if( !md )
-              return ERR_INCOMPATIBLESET;
+              return Mixer::ERR_INCOMPATIBLESET;
            writeVolumeToHW( idx, md->getVolume() );
         }
      }
@@ -215,12 +203,14 @@ int Mixer_SUN::openMixer()
 }
 
 //======================================================================
-// FUNCTION    : Mixer::releaseMixer
+// FUNCTION    : Mixer::close
 // DESCRIPTION : Close the hardware driver.
 //======================================================================
-int Mixer_SUN::releaseMixer()
+int Mixer_SUN::close()
 {
+   m_isOpen = false;
    int l_i_ret = ::close( fd );
+   m_mixDevices.clear();
    return l_i_ret;
 }
 
@@ -233,14 +223,14 @@ QString Mixer_SUN::errorText( int mixer_error )
    QString errmsg;
    switch (mixer_error)
    {
-      case ERR_PERM:
+      case Mixer::ERR_PERM:
          errmsg = i18n(
            "kmix: You do not have permission to access the mixer device.\n"
            "Ask your system administrator to fix /dev/audioctl to allow access."
          );
          break;
       default:
-         errmsg = Mixer::errorText( mixer_error );
+         errmsg = Mixer_Backend::errorText( mixer_error );
    }
    return errmsg;
 }
@@ -422,7 +412,7 @@ bool Mixer_SUN::isRecsrcHW( int devnum )
 //======================================================================
 void Mixer_SUN::VolumeToGainBalance( Volume& volume, uint_t& gain, uchar_t& balance )
 {
-   if ( ( volume.channels() == 1 ) ||
+   if ( ( volume.count() == 1 ) ||
         ( volume[Volume::LEFT] == volume[Volume::RIGHT] ) )
    {
       gain = volume[Volume::LEFT];
@@ -454,7 +444,7 @@ void Mixer_SUN::VolumeToGainBalance( Volume& volume, uint_t& gain, uchar_t& bala
 //======================================================================
 void Mixer_SUN::GainBalanceToVolume( uint_t& gain, uchar_t& balance, Volume& volume )
 {
-   if ( volume.channels() == 1 )
+   if ( volume.count() == 1 )
    {
       volume.setVolume( Volume::LEFT, gain );
    }
