@@ -28,6 +28,8 @@ public slots:
 
 private:
     QLayout* _layoutMDW;
+    QLayout* _layoutEnum;
+    QLayout* _layoutSwitch;
 };
 
 #endif
