@@ -42,20 +42,33 @@
  *
  */
 
-#include "include/wm_config.h"
+#include "include/wm_cdda.h"
 
 #ifdef BUILD_CDDA
  
-static char cddaslave_id[] = "$Id: cddaslave.c,v 1.2 2001/06/05 05:37:25 dfoerste Exp $";
+static char cddaslave_id[] = "$Id: cddaslave.c 257309 2003-10-08 19:37:45Z kernalex $";
 
 #include <stdio.h>
 #include <sys/types.h>
 #include <sys/time.h>
+#include <unistd.h>
+#include <stdlib.h>
+#include <string.h>
+#include "include/wm_struct.h"
 #include "include/wm_cdda.h"
 #include "include/wm_platform.h"
+#include "audio/audio.h"
 
+void send_status(struct cdda_block *);
+
+#define SEND_ACK(b); { (b)->status |= WMCDDA_ACK; send_status(b); }
+#define SEND_STATUS(b); { (b)->status &= ~WMCDDA_ACK; send_status(b); }
+#define SEND_STATUS_ACK(b, s); { (b)->status = ((s) | WMCDDA_ACK); send_status(b); }
+
+int receive_command(struct cdda_device *, struct cdda_block *);
 
 int	playing = 0;		/* Should the CD be playing now? */
+int pausing = 0;
 
 /*
  * Loudness setting, plus the floating volume multiplier and decaying-average
@@ -75,6 +88,12 @@ int		speed = 128;
  */
 FILE		*output = NULL;
 
+/*
+ * These are driverdependent oops
+ *
+ */
+struct audio_oops *oops = NULL;
+
 /*
  * Audio file header format.
  */
@@ -89,6 +108,10 @@ struct auheader {
 };
 
 /* had to change #ifdef to #if   -> see wm_cdda.h */
+#ifdef __FreeBSD__
+/* Phungus not with htonl on FreeBSD */
+#include <sys/param.h>
+#else
 #if WM_BIG_ENDIAN
 # ifndef htonl
 #  define htonl(x) (x)
@@ -96,6 +119,7 @@ struct auheader {
 #else
 extern unsigned long htonl(x);
 #endif
+#endif
 
 void *malloc();
 long cdda_transform();
@@ -104,10 +128,11 @@ long cdda_transform();
  * Send status information upstream.
  */
 void
-send_status(blk)
-	struct cdda_block	*blk;
+send_status(struct cdda_block *blk)
 {
-	write(1, blk, sizeof(*blk));
+  DEBUGLOG("send_status, send %i(%s | %s)\n", blk->status,
+    gen_status(blk->status & WMCDDA_ACK), gen_status(blk->status & ~WMCDDA_ACK));
+  write(1, blk, sizeof(*blk));
 }
 
 /*
@@ -130,163 +155,150 @@ send_status(blk)
  *   Fllllx...          Start saving to a file (length = l, followed by name)
  *   F0000              Stop saving.
  *   Ln			Set loudness level (0-255).
+ *   A                  Pause/Resume
+ *   I                  Get status, current frame
  */
 int
-command(cd_fd, blk)
-	int			cd_fd;
-	struct cdda_block	*blk;
+receive_command(struct cdda_device *dev, struct cdda_block* blk)
 {
-	unsigned char		inbuf[10];
-	char			*filename;
-	int			namelen;
-	struct auheader		hdr;
+  unsigned char	inbuf[10];
+  char          *filename;
+  int             namelen;
+  struct auheader     hdr;
 
-	if (read(0, inbuf, 1) <= 0)	/* Parent died. */
-	{
-		wmcdda_close();
-		wmaudio_close();
-		exit(0);
-	}
+  if (read(0, inbuf, 1) <= 0) {
+    wmcdda_close(dev);
+    oops->wmaudio_close();
+/*    ERRORLOG("cddaslave: parent died, exit\n");*/
+    exit(0);
+  }
 
-	switch (inbuf[0]) {
-	case 'E':
-		playing = 0;
-		wmaudio_stop();
-		wmcdda_close(cd_fd);
-		cd_fd = -1;
-	        blk->status = WMCDDA_ACK;
-	        send_status(blk);
-		break;
-	case 'P':
-		read(0, inbuf, 9);
-		playing = 1;
+  DEBUGLOG("cddaslave: CMD %c\n", inbuf[0]);
 
-		wmaudio_stop();
+  switch (inbuf[0]) {
+  case 'I':
+    if(dev->fd < 0) wmcdda_init(dev, blk);
+    SEND_ACK(blk);
+    break;
+  case 'A': /* pause/resume */
+    if(WMCDDA_PLAYING & blk->status) {
+      oops->wmaudio_stop();
+      SEND_STATUS_ACK(blk, WMCDDA_PAUSED);
+    } else if (WMCDDA_PAUSED & blk->status) {
+      SEND_STATUS_ACK(blk, WMCDDA_PLAYING);
+    } else {
+      SEND_ACK(blk);
+    }
+    break;
+  case 'E':
+    oops->wmaudio_stop();
+    wmcdda_close(dev);
+    SEND_ACK(blk);
+    break;
+  case 'P':
+    read(0, inbuf, 9);
 
-		wmcdda_setup(inbuf[0] * 60 * 75 + inbuf[1] * 75 + inbuf[2],
-			inbuf[3] * 60 * 75 + inbuf[4] * 75 + inbuf[5],
-			inbuf[6] * 60 * 75 + inbuf[7] * 75 + inbuf[8]);
+    wmcdda_setup(inbuf[0] * 60 * 75 + inbuf[1] * 75 + inbuf[2],
+      inbuf[3] * 60 * 75 + inbuf[4] * 75 + inbuf[5],
+      inbuf[6] * 60 * 75 + inbuf[7] * 75 + inbuf[8]);
 
-		wmaudio_ready();
-
-		level = 2500;
-		volume = 1 << 15;
-
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
-		break;
-
-	case 'S':
-		playing = 0;
-		wmaudio_stop();
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
-		blk->status = WMCDDA_STOPPED;
-		send_status(blk);
-		break;
-
-	case 'B':
-		read(0, inbuf, 1);
-		wmaudio_balance(inbuf[0]);
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
-		break;
-
-	case 'V':
-		read(0, inbuf, 1);
-		wmaudio_volume(inbuf[0]);
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
-		break;
-
-	case 'G':
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
-
-		if (playing)
-			blk->status = WMCDDA_PLAYED;
-		else
-			blk->status = WMCDDA_STOPPED;
-		wmaudio_state(blk);
-		send_status(blk);
-		break;
-
-	case 'Q':
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
-		wmcdda_close();
-		wmaudio_close();
-		exit(0);
+    level = 2500;
+    volume = 1 << 15;
 
+    blk->track =  -1;
+    blk->index =  0;
+    blk->minute = inbuf[6];
+    blk->second = inbuf[7];
+    blk->frame  = inbuf[8];
+    SEND_STATUS_ACK(blk, WMCDDA_PLAYING);
+    break;
+  case 'S':
+    oops->wmaudio_stop();
+    SEND_STATUS_ACK(blk, WMCDDA_STOPPED);
+    break;
+  case 'B':
+    read(0, inbuf, 1);
+    if(oops->wmaudio_balance)
+      oops->wmaudio_balance(inbuf[0]);
+    break;
+  case 'V':
+    read(0, inbuf, 1);
+    if(oops->wmaudio_balance)
+      oops->wmaudio_volume(inbuf[0]);
+    break;
+  case 'G':
+    SEND_ACK(blk);
+    if(!oops->wmaudio_state || oops->wmaudio_state(blk) == -1) {
+      blk->volume = -1;
+      blk->balance = 128;
+    }
+    send_status(blk);
+    break;
+  case 'Q':
+    SEND_ACK(blk);
+    wmcdda_close(dev);
+    oops->wmaudio_close();
+    exit(0);
+/*
 	case 's':
 		read(0, inbuf, 1);
 		speed = inbuf[0];
 		wmcdda_speed(speed);
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
+    SEND_STATUS(blk, WMCDDA_ACK);
 		break;
 
 	case 'd':
 		read(0, inbuf, 1);
 		wmcdda_direction(inbuf[0]);
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
+    SEND_STATUS(blk, WMCDDA_ACK);
 		break;
+*/
+  case 'L':
+    read(0, inbuf, 1);
+    loudness = inbuf[0];
+    SEND_ACK(blk);
+    break;
+  case 'F':
+    read(0, &namelen, sizeof(namelen));
+    if (output != NULL) {
+      fclose(output);
+      output = NULL;
+    }
+    if (namelen) {
+      filename = malloc(namelen + 1);
+      if (filename == NULL) {
+        perror("cddaslave");
+        wmcdda_close(dev);
+        oops->wmaudio_close();
+	exit(1);
+      }
 
-	case 'L':
-		read(0, inbuf, 1);
-		loudness = inbuf[0];
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
-		break;
+      read(0, filename, namelen);
+      filename[namelen] = '\0';
+      output = fopen(filename, "w");
+      if (output == NULL)
+        perror(filename);
+      else {
+	/* Write an .au file header. */
+	hdr.magic = htonl(0x2e736e64);
+	hdr.hdr_size = htonl(sizeof(hdr) + 28);
+	hdr.data_size = htonl(~0);
+	hdr.encoding = htonl(3); /* linear-16 */
+	hdr.sample_rate = htonl(44100);
+	hdr.channels = htonl(2);
 
-	case 'F':
-		read(0, &namelen, sizeof(namelen));
-		if (output != NULL)
-		{
-			fclose(output);
-			output = NULL;
-		}
-		if (namelen)
-		{
-			filename = malloc(namelen + 1);
-			if (filename == NULL)
-			{
-				perror("cddaslave");
-				wmcdda_close();
-				wmaudio_close();
-				exit(1);
-			}
+	fwrite(&hdr, sizeof(hdr), 1, output);
+	fwrite("Recorded from CD by WorkMan", 28, 1, output);
+      }
+      free(filename);
+    }
+    SEND_ACK(blk);
+  }
 
-			read(0, filename, namelen);
-			filename[namelen] = '\0';
-			output = fopen(filename, "w");
-			if (output == NULL)
-				perror(filename);
-			else
-			{
-				/* Write an .au file header. */
-				hdr.magic = htonl(0x2e736e64);
-				hdr.hdr_size = htonl(sizeof(hdr) + 28);
-				hdr.data_size = htonl(~0);
-				hdr.encoding = htonl(3);	/* linear-16 */
-				hdr.sample_rate = htonl(44100);
-				hdr.channels = htonl(2);
-
-				fwrite(&hdr, sizeof(hdr), 1, output);
-				fwrite("Recorded from CD by WorkMan", 28, 1,
-					output);
-			}
-
-			free(filename);
-		}
-
-		blk->status = WMCDDA_ACK;
-		send_status(blk);
-	}
-	return(cd_fd);
+  return(dev->fd);
 }
 
+
 /*
  * Transform some CDDA data.
  */
@@ -429,137 +441,129 @@ wmcdda_transform(unsigned char *rawbuf, long buflen, struct cdda_block *block)
 }
 
 
-main(argc, argv)
-	char	**argv;
+int main(int argc, char **argv)
 {
-	int			cd_fd = 3;
-	fd_set			readfd, dummyfd;
-	struct timeval		timeout;
-	char			*cddabuf;
-	long			cddabuflen;
-	struct cdda_block	blockinfo;
-	long			result;
-	int			nfds;
-	char			*devname;
+  fd_set       readfd, dummyfd;
+  struct timeval       timeout;
+  struct cdda_block  blockinfo;
+  long		 	result;
+  int                     nfds;
+  struct cdda_device       dev;
+  const char       *sondsystem;
+  const char      *sounddevice;
+  const char   *sounddevicectl;
 
-	/*
-	 * Device name should be the command-line argument.
-	 */
-	if (argc < 2)
-		devname = "";
-	else
-		devname = argv[1];
+  memset(&blockinfo, 0, sizeof(struct cdda_block));
+  
+  /*
+   * Device name should be the command-line argument.
+   */
+  if (argc < 2)
+    dev.devname = NULL;
+  else
+    dev.devname = argv[1];
 
-	/*
-	 * If we're running setuid root, bump up our priority then lose
-	 * superuser access.
-	 */
-	nice(-14);
-	setuid(getuid());
+  if (argc < 3)
+    sondsystem = "arts";
+  else
+    sondsystem = argv[2];
 
-	FD_ZERO(&dummyfd);
-	FD_ZERO(&readfd);
+  if (argc < 4)
+    sounddevice = NULL;
+  else
+    sounddevice = argv[3];
 
-	timerclear(&timeout);
+  if (argc < 5)
+    sounddevicectl = NULL;
+  else
+    sounddevicectl = argv[3];
 
-	cd_fd = wmcdda_init(&cddabuf, &cddabuflen, cd_fd, devname);
-	if (cd_fd < 0)
-		exit(1);
-	wmaudio_init();
+  DEBUGLOG("cddaslave: called with %s %s %s %s\n",
+    dev.devname?dev.devname:"",
+    sondsystem?sondsystem:"",
+    sounddevice?sounddevice:"",
+    sounddevicectl?sounddevicectl:"");
 
-	blockinfo.status = WMCDDA_ACK;
-	send_status(&blockinfo);
-	blockinfo.status = WMCDDA_STOPPED;
+  /*
+   * If we're running setuid root, bump up our priority then lose
+   * superuser access.
+   */
+  nice(-14);
+  setuid(getuid());
+
+  FD_ZERO(&dummyfd);
+  FD_ZERO(&readfd);
+
+  timerclear(&timeout);
+
+  dev.fd = -1;
+  wmcdda_init(&dev, &blockinfo);
+
+  oops = setup_soundsystem(sondsystem, sounddevice, sounddevicectl);
+  if (!oops) {
+    ERRORLOG("cddaslave: setup_soundsystem failed\n");
+    exit(1);
+  }
+
+  DEBUGLOG("cddaslave: sent first ACK\n");
+  SEND_ACK(&blockinfo);
 
 	/*
 	 * Accept commands as they come in, and play some sound if we're
 	 * supposed to be doing that.
 	 */
-	while (1)
-	{
-		FD_SET(0, &readfd);
+  while (1) {
+    FD_SET(0, &readfd);
 
-		/*
-		 * If we're playing, we don't want select to block.  Otherwise,
-		 * wait a little while for the next command.
-		 */
-		if (playing)
-			timeout.tv_usec = 0;
-		else
-			timeout.tv_usec = 500000;
+    /*
+     * If we're playing, we don't want select to block.  Otherwise,
+     * wait a little while for the next command.
+     */
+    if (playing)
+      timeout.tv_usec = 0;
+    else
+      timeout.tv_usec = 500000;
 
-		nfds = select(1, &readfd, &dummyfd, &dummyfd, &timeout);
+    nfds = select(1, &readfd, &dummyfd, &dummyfd, &timeout);
 
-		if (nfds < 0)	/* Broken pipe; our GUI exited. */
-		{
-			wmcdda_close(cd_fd);
-			wmaudio_close();
-			exit(0);
-		}
+    if (nfds < 0) { /* Broken pipe; our GUI exited. */
+      wmcdda_close(&dev);
+      oops->wmaudio_close();
+      exit(0);
+    }
 
-		if (FD_ISSET(0, &readfd))
-		{
-			/* If this doesn't work, just hope for the best */
-			if(cd_fd == -1)
-			    cd_fd = wmcdda_open(devname);
-			cd_fd = command(cd_fd, &blockinfo);
-			/*
-			 * Process all commands in rapid succession, rather
-			 * than possibly waiting for a CDDA read.
-			 */
-			continue;
-		}
+    if (FD_ISSET(0, &readfd)) {
+      receive_command(&dev, &blockinfo);
+      /*
+       * Process all commands in rapid succession, rather
+       * than possibly waiting for a CDDA read.
+       */
+      continue;
+    }
 		
-		if (playing)
-		{
-			result = wmcdda_read(cd_fd, cddabuf, cddabuflen,
-				&blockinfo);
-			if (result <= 0)
-			{
-				/* Let the output queue drain. */
-				if (blockinfo.status == WMCDDA_DONE)
-				{
-					wmaudio_mark_last();
-					if (wmaudio_send_status())
-					{
-						/* queue drained, stop polling*/
-						playing = 0;
-					}
-				}
-				else
-				{
-					playing = 0;
-					send_status(&blockinfo);
-				}
-			}
-			else
-			{
-				result = wmcdda_normalize(cddabuf, result,
-							&blockinfo);
-				result = wmcdda_transform(cddabuf, result,
-							&blockinfo);
-				if (output)
-					fwrite(cddabuf, result, 1, output);
-				result = wmaudio_convert(cddabuf, result,
-							&blockinfo);
-				if (wmaudio_play(cddabuf, result, &blockinfo))
-				{
-					playing = 0;
-					wmaudio_stop();
-					send_status(&blockinfo);
-				}
-			}
-		}
-		else
-			send_status(&blockinfo);
-	}
-}
+    if ((blockinfo.status & ~WMCDDA_ACK) == WMCDDA_PLAYING) {
+      result = wmcdda_read(&dev, &blockinfo);
+      if (result <= 0 && blockinfo.status != WMCDDA_DONE) {
+        ERRORLOG("cddaslave: wmcdda_read failed\n");
+        blockinfo.status = WMCDDA_STOPPED;
+	send_status(&blockinfo);
+      } else {
+        result = wmcdda_normalize(&dev, &blockinfo);
+        if (output)
+          fwrite(dev.buf, result, 1, output);
 
-#else /* BUILD_CDDA */
+        if (oops->wmaudio_play(dev.buf, dev.buflen, &blockinfo)) {
+          oops->wmaudio_stop();
+          ERRORLOG("cddaslave: wmaudio_play failed\n");
+          blockinfo.status = WMCDDA_STOPPED;
+          send_status(&blockinfo);
+        }
+      }
+    } else
+        send_status(&blockinfo);
+  }
 
-main()
-{
-	exit(0);
+  return 0;
 }
 
 #endif /* BUILD_CDDA */
