@@ -1,7 +1,7 @@
 /*
   Copyright (C) 2002 Rik Hemsley (rikkus) <rik@kde.org>
   Copyright (C) 2002 Benjamin Meyer <ben-devel@meyerhome.net>
-  Copyright (C) 2003-2004 Richard Lrkng <nouseforaname@home.se>
+  Copyright (C) 2003-2004 Richard Lärkäng <nouseforaname@home.se>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -35,6 +35,21 @@ namespace KCDDB
     // Empty.
   }
 
+  CDDB::Result Submit::submit( const CDInfo& cdInfo, const TrackOffsetList &offsetList)
+  {
+    makeDiskData( cdInfo, offsetList );
+
+    if (!validCategory(cdInfo.category))
+      return InvalidCategory;
+
+    KIO::Job* job = createJob(cdInfo);
+
+    if (!job)
+      return UnknownError;
+
+    return runJob(job);
+  }
+
     CDDB::Result
   Submit::parseWrite(  const QString & line )
   {
@@ -50,8 +65,6 @@ namespace KCDDB
   {
     unsigned numTracks = cdInfo.trackInfoList.count();
 
-    diskData_ = "Content-Type: text/plain; charset=\"utf-8\";";
-    diskData_ += "\n";
     diskData_ += "# xmcd\n";
     diskData_ += "#\n";
     diskData_ += "# Track frame offsets:\n";
