@@ -15,7 +15,7 @@
 #define __DITHER2YUV_H
 
  
-#include "../../mmx.h"
+#include "../../mmx/mmx.h"
 #include "../yuvPicture.h"
 
 
