@@ -13,6 +13,9 @@
 
 #include "timeStamp.h"
 
+#include <iostream>
+
+using namespace std;
 
 
 TimeStamp::TimeStamp() {
