@@ -12,6 +12,7 @@
 
 #include "cddaPlayObject_impl.h"
 #include "debug.h"
+#include <iostream>
 
 
 
@@ -25,7 +26,7 @@ CDDAPlayObject_impl::~CDDAPlayObject_impl() {
 
 
 DecoderPlugin* CDDAPlayObject_impl::createPlugin() {
-  cout << "CDDAPlayObject_impl::getPlugin"<<endl;
+    std::cout << "CDDAPlayObject_impl::getPlugin"<<std::endl;
   return new CDDAPlugin();
 }
 
