@@ -1,7 +1,7 @@
 #ifndef __GLOBALVIDEO_H
 #define __GLOBALVIDEO_H
 
-#include <noatun/video.h>
+#include "noatun/video.h"
 
 
 class GlobalVideo : public QWidget
