@@ -6,12 +6,12 @@ This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
-  
+
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
-   
+
 You should have received a copy of the GNU Library General Public License
 along with this library; see the file COPYING.LIB.  If not, write to
 the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
@@ -23,7 +23,9 @@ Boston, MA 02111-1307, USA.
 #include "artsflow.h"
 #include "fft.h"
 #include <stdsynthmodule.h>
-#include <math.h>
+#include <cmath>
+
+#include <iostream>
 
 using namespace std;
 using namespace Arts;
@@ -40,31 +42,28 @@ static void doFft(float combine, float *inBuffer, vector<float> &scope)
 
 	scope.clear();
 
-	unsigned int start=0;
-	unsigned int end=3;
-	
-	float *out_imgIt=out_real;
-	float *out_realIt=out_img;
-	
-	while (start < SAMPLES/2)
+	int previous=0;
+	int index=20;
+
+	while (previous < 2048 && index < 2048)
 	{
-		float xrange(0.0);
-		
-		do
+		int end = int(std::exp(double(index)*combine));
+		float xrange = 0.0;
+
+		while (previous < end)
 		{
-			xrange += (fabs(*out_imgIt) + fabs(*out_realIt))/(float)SAMPLES;
-			++start;
-			out_imgIt++;
-			out_realIt++;
-		} while(start < end);
-		
+			xrange += (std::fabs(out_img[previous]) + std::fabs(out_real[previous]));
+			previous++;
+		}
+
+		xrange /= float(SAMPLES);
 		scope.push_back(xrange);
-		end=(int)((float)(start+1)*combine);
-		++end;
-	}			
+
+		index++;
+	}
 }
 
-		
+
 class FFTScopeStereo_impl : public FFTScopeStereo_skel, public StdSynthModule
 {
 protected:
@@ -72,9 +71,9 @@ protected:
 	vector<float> mScopeRight;
 
 	float mCombine;
-	
+
 	float *mWindow;
-	
+
 	float *mInBufferLeft, *mInBufferRight;
 	unsigned long mInBufferPos;
 
@@ -86,11 +85,11 @@ public:
 		unsigned long i;
 		for(i=0;i<SAMPLES;i++)
 		{
-			float x = (float)i/(float)SAMPLES; 
+			float x = (float)i/(float)SAMPLES;
 			// double it, since we're at half intensity without
 			// adding both channels
 			mWindow[i] = sin(x*DDC_PI)*sin(x*DDC_PI)*2;
-			
+
 			mInBufferLeft[i] = 0;
 			mInBufferRight[i] = 0;
 		}
@@ -136,12 +135,12 @@ public:
 		}
 	}
 
-	FFTScopeStereo_impl()
+	FFTScopeStereo_impl() : mCombine(0.152492)
 	{
 		mWindow = new float[SAMPLES];
 		mInBufferLeft = new float[SAMPLES];
 		mInBufferRight = new float[SAMPLES];
-		
+
 	}
 	~FFTScopeStereo_impl()
 	{
@@ -149,18 +148,18 @@ public:
 		delete [] mInBufferLeft;
 		delete [] mInBufferRight;
 	}
-	
+
 	AutoSuspendState autoSuspend() { return asSuspend; }
 
 };
-	
+
 class FFTScope_impl : public FFTScope_skel, public StdSynthModule
 {
 protected:
 	vector<float> mScope;
 
 	float mCombine;
-	
+
 	float *mWindow;
 	float *mInBuffer;
 	unsigned long mInBufferPos;
@@ -173,7 +172,7 @@ public:
 		unsigned long i;
 		for(i=0;i<SAMPLES;i++)
 		{
-			float x = (float)i/(float)SAMPLES; 
+			float x = (float)i/(float)SAMPLES;
 			mWindow[i] = sin(x*DDC_PI)*sin(x*DDC_PI);
 			mInBuffer[i] = 0;
 		}
@@ -187,7 +186,7 @@ public:
 	{
 		return new vector<float>(mScope);
 	}
-	
+
 	/*
 	  in audio stream inleft, inright;
 	  out audio stream outleft, outright;
@@ -200,7 +199,7 @@ public:
 		float *inleftIt=inleft;
 		float *inrightIt=inright;
 		float *windowIt=mWindow+mInBufferPos;
-		
+
 		for(i=0;i<samples;i++)
 		{
 			*inBufferIt = (*inleftIt + *inrightIt)* (*windowIt);
@@ -225,18 +224,18 @@ public:
 
 	}
 
-	FFTScope_impl() : mCombine(1.1)
+	FFTScope_impl() : mCombine(0.152492)
 	{
 		mWindow = new float[SAMPLES];
 		mInBuffer = new float[SAMPLES];
 	}
-	
+
 	~FFTScope_impl()
 	{
 		delete [] mWindow;
 		delete [] mInBuffer;
 	}
-	
+
 	AutoSuspendState autoSuspend() { return asSuspend; }
 };
 
@@ -248,7 +247,7 @@ protected:
 	int mScopeLength;
 	float *mScopeEnd;
 	float *mCurrent;
-	
+
 public:
 	vector<float> *scope()
 	{
@@ -260,7 +259,7 @@ public:
 		                        (mCurrent - mScope) * sizeof(float));
 		return buf;
 	}
-	
+
 	void buffer(long len)
 	{
 		delete [] mScope;
@@ -269,7 +268,7 @@ public:
 		mScope=new float[len];
 		mScopeEnd=mScope+mScopeLength;
 		mCurrent=mScope;
-		
+
 		memset(mScope, 0, mScopeLength);
 	}
 
@@ -277,7 +276,7 @@ public:
 	{
 		return mScopeLength;
 	}
-	
+
 	void calculateBlock(unsigned long samples)
 	{
 		for (unsigned long i=0; i<samples; ++i)
@@ -289,7 +288,7 @@ public:
 			if (mCurrent>=mScopeEnd)
 				mCurrent=mScope;
 		}
-		
+
 		memcpy(outleft, inleft, sizeof(float)*samples);
 		memcpy(outright, inright, sizeof(float)*samples);
 
@@ -299,14 +298,14 @@ public:
 	{
 		mScope=0;
 		buffer(512);
-	
+
 	}
-	
+
 	~RawScope_impl()
 	{
 		delete [] mScope;
 	}
-	
+
 	AutoSuspendState autoSuspend() { return asSuspend; }
 };
 
@@ -314,7 +313,7 @@ class RawScopeStereo_impl : public RawScopeStereo_skel, public StdSynthModule
 {
 protected:
 	int mScopeLength;
-	
+
 	float *mScopeLeft;
 	float *mScopeEndLeft;
 	float *mCurrentLeft;
@@ -322,7 +321,7 @@ protected:
 	float *mScopeRight;
 	float *mScopeEndRight;
 	float *mCurrentRight;
-	
+
 public:
 	vector<float> *scopeLeft()
 	{
@@ -345,7 +344,7 @@ public:
 		       (mCurrentRight - mScopeRight) * sizeof(float));
 		return buf;
 	}
-	
+
 	void buffer(long len)
 	{
 		delete [] mScopeRight;
@@ -358,7 +357,7 @@ public:
 		mScopeEndLeft=mScopeLeft+mScopeLength;
 		mCurrentRight=mScopeRight;
 		mCurrentLeft=mScopeLeft;
-		
+
 		memset(mScopeRight, 0, mScopeLength);
 		memset(mScopeLeft, 0, mScopeLength);
 	}
@@ -367,7 +366,7 @@ public:
 	{
 		return mScopeLength;
 	}
-	
+
 	void calculateBlock(unsigned long samples)
 	{
 		for (unsigned long i=0; i<samples; ++i)
@@ -389,7 +388,7 @@ public:
 			if (mCurrentRight>=mScopeEndRight)
 				mCurrentRight=mScopeRight;
 		}
-		
+
 		memcpy(outleft, inleft, sizeof(float)*samples);
 		memcpy(outright, inright, sizeof(float)*samples);
 	}
@@ -398,9 +397,9 @@ public:
 	{
 		mScopeLeft=mScopeRight=0;
 		buffer(512);
-	
+
 	}
-	
+
 	~RawScopeStereo_impl()
 	{
 		delete [] mScopeRight;
