@@ -1,16 +1,16 @@
 // Copyright (c) 2000-2001 Charles Samuels <charles@kde.org>
 // Copyright (c) 2000-2001 Neil Stevens <multivac@fcmail.com>
-// 
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in
 // all copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
@@ -20,11 +20,12 @@
 
 #include <noatun/app.h>
 
+#include <kdebug.h>
 #include <kdialog.h>
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kmessagebox.h>
-#include <ktabctl.h>
+#include <qtabwidget.h>
 #include <qheader.h>
 #include <qlabel.h>
 #include <qlayout.h>
@@ -104,7 +105,7 @@ void PluginListView::stateChanged(PluginListItem *item, bool b)
 	{
 		count++;
 		emit stateChange(item, b);
-		
+
 		if(hasMaximum && count > max)
 		{
 			// Find a different one and turn it off
@@ -136,15 +137,15 @@ void PluginListView::stateChanged(PluginListItem *item, bool b)
 }
 
 Plugins::Plugins(QObject *_parent)
-	: CModule(i18n("Plugins"), i18n("Choose Your Plugins"), "gear", _parent)
+	: CModule(i18n("Plugins"), i18n("Select Your Plugins"), "gear", _parent)
 	, shown(false)
 {
-	(new QVBoxLayout(this))->setAutoAdd(true); 
-	KTabCtl *tabControl = new KTabCtl(this);
+	(new QVBoxLayout(this))->setAutoAdd(true);
+	QTabWidget *tabControl = new QTabWidget(this,"tabControl");
 
 	QFrame *interfaceTab = new QFrame(tabControl);
-	(new QVBoxLayout(interfaceTab, KDialog::marginHint(), KDialog::spacingHint()))->setAutoAdd(true); 
-	(void)new QLabel(i18n("<b>Choose one or more interfaces to use:</b>"), interfaceTab);
+	(new QVBoxLayout(interfaceTab, KDialog::marginHint(), KDialog::spacingHint()))->setAutoAdd(true);
+	(void)new QLabel(i18n("<b>Select one or more interfaces to use:</b>"), interfaceTab);
 	// At least one interface is required
 	interfaceList = new PluginListView(1, interfaceTab);
 	interfaceList->addColumn(i18n("Name"));
@@ -153,10 +154,10 @@ Plugins::Plugins(QObject *_parent)
 	interfaceList->addColumn(i18n("License"));
 	connect(interfaceList, SIGNAL(stateChange(PluginListItem *, bool)), this, SLOT(stateChange(PluginListItem *, bool)));
 	tabControl->addTab(interfaceTab, i18n("&Interfaces"));
-	
+
 	QFrame *playlistTab = new QFrame(tabControl);
-	(new QVBoxLayout(playlistTab, KDialog::marginHint(), KDialog::spacingHint()))->setAutoAdd(true); 
-	(void)new QLabel(i18n("<b>Choose one playlist to use:</b>"), playlistTab);
+	(new QVBoxLayout(playlistTab, KDialog::marginHint(), KDialog::spacingHint()))->setAutoAdd(true);
+	(void)new QLabel(i18n("<b>Select one playlist to use:</b>"), playlistTab);
 	// Exactly one playlist is required
 	playlistList = new PluginListView(1, 1, playlistTab);
 	playlistList->addColumn(i18n("Name"));
@@ -167,8 +168,8 @@ Plugins::Plugins(QObject *_parent)
 	tabControl->addTab(playlistTab, i18n("&Playlist"));
 
 	QFrame *visTab = new QFrame(tabControl);
-	(new QVBoxLayout(visTab, KDialog::marginHint(), KDialog::spacingHint()))->setAutoAdd(true); 
-	(void)new QLabel(i18n("<b>Choose any visualizations to use:</b>"), visTab);
+	(new QVBoxLayout(visTab, KDialog::marginHint(), KDialog::spacingHint()))->setAutoAdd(true);
+	(void)new QLabel(i18n("<b>Select any visualizations to use:</b>"), visTab);
 	visList = new PluginListView(0, visTab);
 	visList->addColumn(i18n("Name"));
 	visList->addColumn(i18n("Description"));
@@ -179,8 +180,8 @@ Plugins::Plugins(QObject *_parent)
 
 	// Other plugins are not restricted
 	QFrame *otherTab = new QFrame(tabControl);
-	(new QVBoxLayout(otherTab, KDialog::marginHint(), KDialog::spacingHint()))->setAutoAdd(true); 
-	(void)new QLabel(i18n("<b>Choose any other plugins to use:</b>"), otherTab);
+	(new QVBoxLayout(otherTab, KDialog::marginHint(), KDialog::spacingHint()))->setAutoAdd(true);
+	(void)new QLabel(i18n("<b>Select any other plugins to use:</b>"), otherTab);
 	otherList = new PluginListView(0, otherTab);
 	otherList->addColumn(i18n("Name"));
 	otherList->addColumn(i18n("Description"));
@@ -268,7 +269,7 @@ void Plugins::removePlugin(const NoatunLibraryInfo &info)
 
 	// Here are the ones loaded
 	QValueList<NoatunLibraryInfo> loaded = napp->libraryLoader()->loaded();
-	
+
 	// Add the ones marked for loading
 	for(QStringList::ConstIterator i = mAdded.begin(); i != mAdded.end(); ++i)
 		loaded.append(loader.getInfo(*i));
@@ -339,20 +340,51 @@ void Plugins::save()
 	LibraryLoader &loader = *(napp->libraryLoader());
 
 	// Load the plugins the user added
-	for(QStringList::Iterator i = mAdded.begin(); i != mAdded.end(); ++i)
-		loader.add(*i);
+	//loader.loadAll(mAdded);
+
+	QString oldPlaylist, newPlaylist;
+
+	// first load all non playlist things
+	for (QStringList::Iterator i = mAdded.begin(); i != mAdded.end(); ++i)
+	{
+		NoatunLibraryInfo info = loader.getInfo(*i);
+		if(info.type != "playlist")
+			loader.loadAll(QStringList(*i));
+		else
+			newPlaylist = (*i);
+	}
 
 	// Remove the plugins the user removed
 	for (QStringList::Iterator i = mDeleted.begin(); i != mDeleted.end(); ++i)
-		loader.remove(*i);
+	{
+		NoatunLibraryInfo info = loader.getInfo(*i);
+		if(info.type != "playlist")
+			loader.remove(*i);
+		else
+			oldPlaylist = *i;
+	}
+
+	// Loading normal plugins works the other way round!
+	// If you unload a playlist it sets the global playlist pointer to NULL,
+	// that also means you cannot first load the new and then unload the old one.
+	if(!newPlaylist.isEmpty() && !oldPlaylist.isEmpty())
+	{
+		kdDebug(66666) << k_funcinfo << "Unloading " << oldPlaylist << endl;
+		loader.remove(oldPlaylist);
+		kdDebug(66666) << k_funcinfo << "Loading " << oldPlaylist << endl;
+		loader.loadAll(QStringList(newPlaylist));
+	}
+
 
 	// Round up the ones that weren't loaded right now, for saving in the configuration
 	QStringList specList(mAdded);
 
 	QValueList<NoatunLibraryInfo> loaded = loader.loaded();
 	for(QValueList<NoatunLibraryInfo>::Iterator i = loaded.begin(); i != loaded.end(); ++i)
+	{
 		if(!specList.contains((*i).specfile) && loader.isLoaded((*i).specfile))
-				specList += (*i).specfile;
+			specList += (*i).specfile;
+	}
 
 	// Now we actually save
 	loader.setModules(specList);
