@@ -4,6 +4,7 @@
 #include <qstring.h>
 #include <qpixmap.h>
 #include <qbitmap.h>
+#include <qfont.h>
 
 class KJLoader;
 
@@ -21,7 +22,13 @@ public:
 	int fontSpacing() const {return mSpacing;}
 	bool isTransparent() const {return mTransparent;}
 
+	// !!! Call if you changed the systemfont !!!
+	void recalcSysFont(void);
+
 protected:
+	QPixmap drawSysFont(const QCString &s, int wide, const QPoint &pos=QPoint(0,0)) const;
+	QPixmap drawPixmapFont(const QCString &, int, const QPoint &pos=QPoint(0,0)) const;
+
 	void drawCharacter(QPixmap *dev, QBitmap *devMask, const QPoint &to, char c) const;
 	QPoint charSource(char c) const;
 
@@ -34,6 +41,10 @@ private:
 	bool mTransparent; // indicates wether there's transparency
 	const char *mString[3];
 	char mNullChar;
+	QFontMetrics *sysFontMetrics;
+	QFont sysFont;
+	QColor sysFontColor;
+	bool mUseSysFont;
 };
 
 #endif
