@@ -80,7 +80,7 @@ bool WaSkinModel::load(QString skinDir)
 
     QDir dir(skinDir);
 
-    if (findFile(dir, "main.bmp") == "") {
+    if (findFile(dir, "main.bmp").isEmpty()) {
         // Ack, our skin dir doesn't exist, fall back to the default
         dir = QDir(KGlobal::dirs()->findDirs("data", "noatun/skins/winamp/" + WaSkin::defaultSkin())[0]);
         success = false;
@@ -160,7 +160,7 @@ int WaSkinModel::getPixmap(const QDir &dir, QString fname,
     }
 
     // now the filename mapping 1.8x -> 2.0
-    if (fname.upper() == "volume.bmp") 
+    if (fname == "volume.bmp") 
 	return WaSkinModel::getPixmap(dir, QString("volbar.bmp"), target);
 
     if (fname == "numbers.bmp") 
